
;TV's server version 3.10
;Copyright (c) 2014 - 2019, Tycho Veltmeijer.
;All rights reserved.

;TODO:
;-see StrToIntWithBaseAsTerminator

;----------------------------------------------------
;                      strlen
;
;input:
;    1: Pointer to ASCII string
;
;output: (R/E)AX = size of string
;
;----------------------------------------------------
if used strlen
strlen:
  call denter

  xor ecx,ecx  ; zero ECX

  dec pecx     ; (R/E)CX => -1  (scasb will be repeated for 0xFFFFFFFF(FFFFFFFF) times, or until zero byte is found)
  xor eax,eax  ; byte to find with scasb => '\0'
  mov pedi,pdword [pebp+(psize*2)] ;(R/E)DI => pointer to ASCII string
  mov pedx,pedi ; (R/E)DX => copy of pointer to ASCII string

  test pedi,pedi
  jz _endStrlen    ;jump if pointer is zero

  repnz scasb   ;loop string until zero byte is found

endStrlen:
  sub pedi,pedx ; current pos of zero byte - begin of string = size of string
  mov peax,pedi ;(R/E)AX => size of string
  dec peax

_endStrlen:
  call dleave
  ret (psize*1)
end if

;----------------------------------------------------
;                      strlenW
;
;input:
;    1: Pointer to UNICODE string
;
;output: (R/E)AX = size of string IN BYTES!!!
;
;----------------------------------------------------
if used strlenW
strlenW:
  call denter

  xor ecx,ecx  ; zero ECX

  dec pecx     ; (R/E)CX => -1  (scasb will be repeated for 0xFFFFFFFF(FFFFFFFF) times, or until zero byte is found)
  xor eax,eax  ; byte to find with scasb => '\0'
  mov pedi,pdword [pebp+(psize*2)] ;(R/E)DI => pointer to UNICODE string
  mov pedx,pedi ; (R/E)DX => copy of pointer to UNICODE string

  test pedi,pedi
  jz _endStrlenW    ;jump if pointer is zero

  repnz scasw   ;loop string until zero word is found

endStrlenW:
  sub pedi,pedx ; current pos of zero byte - begin of string = size of string
  mov peax,pedi ;(R/E)AX => size of string
  sub peax,2

_endStrlenW:
  call dleave
  ret (psize*1)
end if

;----------------------------------------------------
;                      strcpy
;This function copies source string to pointer destination.
;
;input:
;    1: Pointer to destination ASCII string
;    2: Pointer to source ASCII string
;
;output: none
;
;----------------------------------------------------
if used strcpy
strcpy:
  call denter
        
  mov pedi,pdword [pebp+(psize*2)]
  mov pesi,pdword [pebp+(psize*3)]
  cld
        
  StartCpy:
    lodsb
    stosb
    test al,al
  jnz StartCpy

  call dleave
  ret (psize*2)
end if

;----------------------------------------------------
;                      strcpyn
;This function copies source string to pointer destination with specific size.
;
;input:
;    1: Pointer to destination ASCII string
;    2: Pointer to source ASCII string
;    3: Size to copy source to destination
;
;output: none
;
;----------------------------------------------------
if used strcpyn
strcpyn:
  call denter

  mov pecx,pdword [pebp+(psize*4)]       ;ECX = size
  mov pedi,pdword [pebp+(psize*2)]       ;EDI= destination
  mov pesi,pdword [pebp+(psize*3)]       ;ESI = source
  rep movsb                              ;Copy [ESI] to [EDI] with size ECX

  call dleave
  ret (psize*3)
end if

;----------------------------------------------------
;                      strcpyW
;This function copies source unicode string to pointer unicode destination.
;
;input:
;    1: Pointer to destination Unicode string
;    2: Pointer to source Unicode string
;
;output: none
;
;----------------------------------------------------
if used strcpyW
strcpyW:
  call denter
        
  mov pedi,pdword [pebp+(psize*2)]
  mov pesi,pdword [pebp+(psize*3)]
  xor ax,ax
  cld
        
  call InternalStrcpyW

  call dleave
  ret (psize*2)
end if

;----------------------------------------------------
;                      InternalStrcpyW
;This function copies source unicode string to pointer unicode destination.
;
;input:
;    (R/E)DI: Pointer to destination Unicode string
;    (R/E)SI: Pointer to source unicode string
;
;output: (R/E)DI => pointer to last converted byte + 1
;        (R/E)DI => Pointer to source unicode string + size(Pointer to source unicode string)
;
;
;----------------------------------------------------
if used InternalStrcpyW
InternalStrcpyW:
  lodsw
  stosw
  test al,al
jnz InternalStrcpyW

  ret
end if

;----------------------------------------------------
;                      InternalStrcpyWCaseHigh
;This function copies source unicode string to pointer unicode destinationand higher low characters.
;
;input:
;    (R/E)DI: Pointer to destination Unicode string
;    (R/E)SI: Pointer to source unicode string
;
;output: (R/E)DI => pointer to last converted byte + 1
;        (R/E)DI => Pointer to source unicode string + size(Pointer to source unicode string)
;
;
;----------------------------------------------------
if used InternalStrcpyWCaseHigh
InternalStrcpyWCaseHigh:
  lodsw
  test ah,ah        ;don;t higher the case if the higher byte in the unicode character is set.
  jnz dontHigherCase
    call SetHighCase
  dontHigherCase:
  stosw
  test al,al
jnz InternalStrcpyWCaseHigh

  ret
end if

;----------------------------------------------------
;                      strcpyAW
;This function copies source ASCII string to pointer unicode destination.
;
;input:
;    1: Pointer to destination Unicode string
;    2: Pointer to source ASCII string
;
;output: none
;
;----------------------------------------------------
if used strcpyAW
strcpyAW:
  call denter
        
  mov pedi,pdword [pebp+(psize*2)]
  mov pesi,pdword [pebp+(psize*3)]
  xor ax,ax
  cld
        
  call InternalStrcpyAW

  call dleave
  ret (psize*2)
end if

;----------------------------------------------------
;                      InternalStrcpyAW
;This function copies source ASCII string to pointer unicode destination.
;
;input:
;    (R/E)DI: Pointer to destination Unicode string
;    (R/E)SI: Pointer to source ASCII string
;
;output: (R/E)DI => pointer to last converted byte + 1
;        (R/E)SI => Pointer to source ASCII string + size(Pointer to source ASCII string)
;
;
;----------------------------------------------------
if used InternalStrcpyAW
InternalStrcpyAW:
  lodsb
  stosw
  test al,al
jnz InternalStrcpyAW

  ret
end if

;----------------------------------------------------
;                      InternalStrcpyAWCaseHigh
;This function copies source ASCII string to pointer unicode destination.
;
;input:
;    (R/E)DI: Pointer to destination Unicode string
;    (R/E)SI: Pointer to source ASCII string
;
;output: (R/E)DI => pointer to last converted byte + 1
;        (R/E)SI => Pointer to source ASCII string + size(Pointer to source ASCII string)
;
;
;----------------------------------------------------
if used InternalStrcpyAWCaseHigh
InternalStrcpyAWCaseHigh:
  lodsb
  call SetHighCase
  stosw
  test al,al
jnz InternalStrcpyAWCaseHigh

  ret
end if

;----------------------------------------------------
;                      strcpyWA
;This function copies source unicode string to pointer ASCII destination.
;
;input:
;    1: Pointer to destination ASCII string
;    2: Pointer to source UNICODE string
;
;output: none
;
;----------------------------------------------------
if used strcpyWA
strcpyWA:
  call denter
        
  mov pedi,pdword [pebp+(psize*2)]
  mov pesi,pdword [pebp+(psize*3)]
  cld
        
  call InternalStrcpyWA

  call dleave
  ret (psize*2)
end if

;----------------------------------------------------
;                      InternalStrcpyWA
;This function copies source unicode string to pointer ASCII destination.
;
;input:
;    (R/E)DI: Pointer to destination unicode string
;    (R/E)SI: Pointer to source ASCII string
;
;output: (R/E)DI => pointer to last converted byte + 1
;        (R/E)SI => Pointer to source ASCII string + size(Pointer to source ASCII string)
;
;
;----------------------------------------------------
if used InternalStrcpyWA
InternalStrcpyWA:
  lodsw
  stosb
  test ax,ax
jnz InternalStrcpyWA
  ret
end if

;----------------------------------------------------
;                      InternalStrcpyWACaseHigh
;This function copies source unicode string to pointer ASCII destination and higher low characters.
;
;input:
;    (R/E)DI: Pointer to destination unicode string
;    (R/E)SI: Pointer to source ASCII string
;
;output: (R/E)DI => pointer to last converted byte + 1
;        (R/E)SI => Pointer to source ASCII string + size(Pointer to source ASCII string)
;
;
;----------------------------------------------------
if used InternalStrcpyWACaseHigh
InternalStrcpyWACaseHigh:
  lodsw
    call SetHighCase
  stosb
  test ax,ax
jnz InternalStrcpyWACaseHigh
  ret
end if

;----------------------------------------------------
;                      strcmp
;This function compairs string 1 to string 2 (case-insentitive)
;
;input:
;    1: Pointer to ASCII string 1
;    2: Pointer to ASCII string 2
;
;output: (R/E)AX: 0 = strings match
;                 1 = strings do not match
;
;
;output: Zero flag:  set = strings are equal
;                    not set = strings are not equal
;----------------------------------------------------
if used strcmp
strcmp:
  call denter

  ;(R/E)DI =>string 1; (R/E)DI =>string 2
  mov pedi,pdword [pebp+(psize*2)]
  mov pesi,pdword [pebp+(psize*3)]
  xor ecx,ecx ; ecx=0 (return value)
  mov al,1 ;set byte to one
  
  StartCmpLoop:
    ;check for terminating zero byte
    test al,al
    jz endStrCmp

    ;AL=>byte [(R/E)SI]; (R/E)SI++;
    lodsb
  
    inc pedi ;string1++;

    ;check if byte string 1 matches string 2
    cmp al,byte [pedi-0x1]
    je StartCmpLoop

    ;Do a case sensitive match
    push StartCmpLoop ;jump addres if match
    call SetCase

  errStrCmp: ;The strings do not match
  inc ecx

  endStrCmp: ;The strings do match
  mov eax,ecx

  call dleave
  test eax,eax ;set return flags
  ret (psize*2)
end if

;----------------------------------------------------
;                      strcmp
;This function compairs string 1 to string 2 (case-insentitive)
;
;input:
;    1: Pointer to ASCII string 1
;    2: Pointer to ASCII string 2
;
;output: (R/E)AX: 0 = strings match
;                 1 = strings do not match
;
;
;output: Zero flag:  set = strings are equal
;                    not set = strings are not equal
;----------------------------------------------------
if used strcmpW
strcmpW:
  call denter

  ;(R/E)DI =>string 1; (R/E)DI =>string 2
  mov pedi,pdword [pebp+(psize*2)]
  mov pesi,pdword [pebp+(psize*3)]
  xor ecx,ecx ; ecx=0 (return value)
  mov ax,1 ;set byte to one
  
  StartCmpWLoop:
    ;check for terminating zero byte
    test ax,ax
    jz endStrCmpW

    ;AL=>byte [(R/E)SI]; (R/E)SI++;
    lodsw
  
    add pedi,2 ;string1++;

    ;check if byte string 1 matches string 2
    cmp ax,word [pedi-0x1]
    je StartCmpWLoop

    ;setcase does not handle words > sizeof(char)
    test ah,ah
    jnz errStrCmpW

    ;Do a case sensitive match
    push StartCmpLoop ;jump addres if match
    call SetCase

  errStrCmpW: ;The strings do not match
  inc ecx

  endStrCmpW: ;The strings do match
  mov eax,ecx

  call dleave
  test eax,eax ;set return flags
  ret (psize*2)
end if


;----------------------------------------------------
;                      strcmpCaseSensitive
;This function compairs string 1 to string 2 (case-sentitive)
;
;input:
;    1: Pointer to ASCII string 1
;    2: Pointer to ASCII string 2
;
;output: (R/E)AX: 0 = strings match
;                 1 = strings do not match
;
;
;output: Zero flag:  set = strings are equal
;                    not set = strings are not equal
;----------------------------------------------------
if used strcmpCaseSensitive
strcmpCaseSensitive:
  call denter

  ;(R/E)DI =>string 1; (R/E)DI =>string 2
  mov pedi,pdword [pebp+(psize*2)]
  mov pesi,pdword [pebp+(psize*3)]
  xor ecx,ecx ; ecx=0 (return value)
  
  StartCmpCaseLoop:
    lodsb
    scasb
    jz endStartCmpCaseLoop

    test al,al
  jnz StartCmpCaseLoop

  errStrcmpCaseSensitive: ;The strings do not match
  inc ecx

  endStartCmpCaseLoop: ;The strings do match
  mov eax,ecx

  call dleave
  test eax,eax ;set return flags
  ret (psize*2)
end if

;----------------------------------------------------
;                      strcmpCaseSensitive
;This function compairs string 1 to string 2 (case-sentitive)
;
;input:
;    1: Pointer to ASCII string 1
;    2: Pointer to ASCII string 2
;
;output: (R/E)AX: 0 = strings match
;                 1 = strings do not match
;
;
;output: Zero flag:  set = strings are equal
;                    not set = strings are not equal
;----------------------------------------------------
if used strcmpCaseSensitiveW
strcmpCaseSensitiveW:
  call denter

  ;(R/E)DI =>string 1; (R/E)DI =>string 2
  mov pedi,pdword [pebp+(psize*2)]
  mov pesi,pdword [pebp+(psize*3)]
  xor ecx,ecx ; ecx=0 (return value)
  
  StartCmpCaseLoopW:
    lodsw
    scasw
    jz endStrcmpCaseSensitiveW

    test ax,ax
  jnz StartCmpCaseLoopW

  errStrcmpCaseSensitiveW: ;The strings do not match
  inc ecx

  endStrcmpCaseSensitiveW: ;The strings do match
  mov eax,ecx

  call dleave
  test eax,eax ;set return flags
  ret (psize*2)
end if


;----------------------------------------------------
;                comapairMemCaseSen
;This function compairs memory 1 to memory 2 (case-insentitive) with specific size.
;
;input:
;    1: Pointer to ASCII memory 1
;    2: Pointer to ASCII memory 2
;    3: Size to compaire
;
;output: (R/E)AX: 0 = strings match
;                 1 = strings do not match
;
;output: Zero flag:  set = strings are equal
;                    not set = strings are not equal
;----------------------------------------------------
if used comapairMemCaseSen ;| used bufcmp
comapairMemCaseSen:
;bufcmp:
  call denter

  ;(R/E)DI =>buf 1; (R/E)DI =>buf 2; (R/E)CX = size
  mov pedi,pdword [pebp+psize*2]
  mov pesi,pdword [pebp+psize*3]
  mov pecx,pdword [pebp+psize*4]
  xor edx,edx ; ECX = 0 (return value)

  startComapairMemCaseSenLoop:
    jecxz endComapairMemCaseSen
    ;compaire [edi] to [esi] with size ECX
    repz cmpsb
    jz endComapairMemCaseSen  ;jump if buffers match

    ;check on lower-higher case
    mov al,byte [pesi-1]

    ;Do a case sensitive match
    push startComapairMemCaseSenLoop ;jump addres if match
    call SetCase

errComapairMemCaseSen:
  inc edx     ;return value = 1  (and unset zero flag)

endComapairMemCaseSen:
  mov eax,edx ;return the return value

  call dleave

  test eax,eax
  ret (psize*3)
end if

;----------------------------------------------------
;                comapairMem
;This function compairs memory 1 to memory 2 with specific size.
;
;input:
;    1: Pointer to memory 1
;    2: Pointer to memory 2
;    3: Size to compaire
;
;output: Zero flags: set = strings are equal
;                    not set = strings are not equal
;
;----------------------------------------------------
if used comapairMem
comapairMem:
  push pebp
  mov pebp,pesp

  push pedi
  push pesi
  push pecx

  ;(R/E)DI =>buf 1; (R/E)DI =>buf 2; (R/E)CX = size
  mov pedi,pdword [pebp+psize*2]
  mov pesi,pdword [pebp+psize*3]
  mov pecx,pdword [pebp+psize*4]

    ;test pecx,pecx
    jpcxz endComapairMem

    ;compaire [edi] to [esi] with size ECX
    repz cmpsb
    ;jz endBufcmp  ;jump if buffers match

  endComapairMem:
  pop pecx
  pop pesi
  pop pedi

  mov pesp,pebp
  pop pebp
  ret (psize*3)
end if

;----------------------------------------------------
;                      SetCase
;This function lowers or highers a byte it's case
;
;input:
;    1:Pointer to jump to when the higher/lower byte match
;    AL:ASCII byte
;    byte [pedi-0x1]: byte to compair the higher or lower byte with
;
;output: AL: Higher, lowered or same byte as input
;
;corrupts: AH
;
;----------------------------------------------------
if used SetCase
SetCase:
      ;add the difference between high and low case  (0x20)
      mov ah,al
      add ah,0x20

      ;is char between A and Z?
      cmp al,0x40
      jna isnotHighCase

      cmp al,0x5B
      jnae caseChanged

    isnotHighCase:
      ;is char between  and ?
      cmp al,0xBF
      jna isnotHighSpecialCase

      cmp al,0xDF
      jnae caseChanged

    isnotHighSpecialCase:
      sub ah,0x40 ;set case to high

      ;is char between a and z?
      cmp al,0x60
      jna isNotLowCase

      cmp al,0x7B
      jnae caseChanged

    isNotLowCase:
      ;is char between  and ?
      cmp al,0x0DF
      jnae notCaseProblem

      ;-cmp al,0xFF
      ;-jnb notCaseProblem  ;the char is not high or low case

    caseChanged:
      ;check again, but now with the low or high case
      cmp ah,byte [pedi-0x1]
      jne notCaseProblem

      add pesp,psize*2
      jmp pdword [pesp-psize]


    notCaseProblem:

    ret psize
end if

;----------------------------------------------------
;                      SetHighCase
;This function lowers or highers a byte it's case
;
;input:
;    AL:ASCII byte
;
;output: AL: Higher or same byte as input
;
;
;----------------------------------------------------
if used SetHighCase
SetHighCase:
      test al,al
      jz endSetHighCase

      ;is char between A and Z?
      cmp al,0x40
      jna isnotHighCase1

      cmp al,0x5B
      jnae endSetHighCase

    isnotHighCase1:
      ;is char between  and ?
      cmp al,0xBF
      jna isnotHighSpecialCase1

      cmp al,0xDD
      jnae endSetHighCase

    isnotHighSpecialCase1:
      ;is char between a and z?
      cmp al,0x60
      jna isNotLowCase1

      cmp al,0x7B
      jnae setCaseHigher

    isNotLowCase1:
      ;is char between  and ?
      cmp al,0x0DF
      jnae endSetHighCase;setCaseHigher

      ;-cmp al,0xFF
      ;-jnb endSetHighCase  ;the char is not high or low case

    setCaseHigher:
      sub al,0x20 ;set case to high (add the difference between high and low case  (0x20))

    endSetHighCase:
    ret
end if

;----------------------------------------------------
;                      copyStringA
;This function copies a string to a new allocated memory
;
;input:
;    1:Pointer to string to copy
;
;output: (R/E)AX: pointer to memory with copied string
;
;----------------------------------------------------
if used copyStringA
copyStringA:
  call denter

  ;(R/E)SI => String to copy
  mov pesi,pdword [pebp+(psize*2)]

  ;Calculate string to copy
  push pesi
  call strlen
  mov pecx,peax

  ;make sure you request at least 1 byte (if the string is empty eax will be zero and GetMemLocation will fail)
  inc peax

  ;Allocate new memory pointer
  push peax
  call GetMemLocation

  ;return a empty pointer if the input string is zero
  test pesi,pesi
  jz endCopyStringA

  push peax  ;save new memory pointer

  ;copy the string to the new memory pointer
  mov pedi,peax
  rep movsb

  pop peax   ;restore new memory pointer

endCopyStringA:
  call dleave

  ret (psize*1)
end if

;----------------------------------------------------
;                      copyStringW
;This function copies a Wide Char string to a new allocated memory
;
;input:
;    1:Pointer to Wide Char string to copy
;
;output: (R/E)AX: pointer to memory with copied string
;
;----------------------------------------------------
if used copyStringW
copyStringW:
  call denter

  ;(R/E)SI => String to copy
  mov pesi,pdword [pebp+(psize*2)]

  ;Calculate string to copy
  push pesi
  call strlenW
  mov ecx,eax
  shr ecx,1 ;get size in words, instead of bytes (ECX=ECX/2)

  ;make sure you request at least 2 bytes (if the string is empty eax will be zero and GetMemLocation will fail)
  add peax,2

  ;Allocate new memory pointer
  push peax
  call GetMemLocation

  ;return a empty pointer if the input string is zero
  test pesi,pesi
  jz endCopyStringW

  push peax  ;save new memory pointer

  ;copy the string to the new memory pointer
  mov pedi,peax
  rep movsw

  pop peax   ;restore new memory pointer

endCopyStringW:
  call dleave

  ret (psize*1)
end if

;----------------------------------------------------
;                      copyBuffer
;This function copies a buffer to a new allocated memory
;
;input:
;    1:Pointer to buffer to copy
;    2:size of buffer to copy
;
;output: (R/E)AX: pointer to memory with copied buffer
;
;----------------------------------------------------
if used copyBuffer
copyBuffer:
  call denter

  ;(R/E)SI => Buffer to copy; (R/E)CX => Size
  mov pesi,pdword [pebp+(psize*2)]
  mov pecx,pdword [pebp+(psize*3)]

  test pecx,pecx
  cmovz peax,pesi
  jz endCopyBuffer

  ;Allocate new memory pointer
  push pecx
  call GetMemLocation

  push peax  ;save new memory pointer

  ;copy buffer to the new memory pointer
  mov pedi,peax
  rep movsb

  pop peax   ;restore new memory pointer

endCopyBuffer:
  call dleave

  ret (psize*2)
end if

;----------------------------------------------------
;                      StrToInt
;Converts an ASCII format number to an integer
;
;input:
;    1:source ASCII string
;    2:base for conversion
;
;output: (R/E)AX => integer
;
;----------------------------------------------------
if used StrToInt
StrToInt:
  call denter
  mov pedi,pdword [pebp+(psize*2)] ;(R/E)DI => source ASCII string
  mov pebx,pdword [pebp+(psize*3)] ;(R/E)BX => base for conversion

  xor peax,peax  ;Zero output value
  xor pedx,pedx  ;Zero (R/E)DX
  .loop:
    ;Get digit from string and test on terminating zero byte
    mov dl,byte [pedi]
    test dl,dl
    jz .endx

    ;multiply result by base
    imul peax,pebx

    ;substract ASCII char '0'
    sub dl,'0'

    ;substract the distance between the char '9' and ALPHABETIC char 'A' if nececery
    cmp dl,9
    jbe .ok
    sub dl,7
  .ok:

    ;add the converted digit to the result
    add peax,pedx
    inc pedi ;seek to next char in string
    jmp .loop
  .endx:
  call dleave
  ret (psize*2)
end if

;----------------------------------------------------
;                      StrToIntQ
;Converts an ASCII format number to an 64-bit integer in 32-bit mode
;
;input:
;    1:source ASCII string
;    2:base for conversion
;    3:pointer to 64-bit integer
;
;output: none
;
;----------------------------------------------------
if used StrToIntQ
StrToIntQ:
  call denter
  mov pedi,pdword [pebp+(psize*2)] ;(R/E)DI => source ASCII string
  mov pecx,pdword [pebp+(psize*3)] ;(R/E)BX => base for conversion
  mov pesi,pdword [pebp+(psize*4)] ;(R/E)SI => pointer to 64-bit integer

  xor pedx,pedx  ;Zero (R/E)DX
  ;zero 64-bit integer
  mov dword [pesi],edx
  mov dword [pesi+4],edx

  .loop:
    ;Get digit from string and test on terminating zero byte
    mov dl,byte [pedi]
    test dl,dl
    jz .endx

    ;multiply result by base
    push pesi
    call MulQword

    ;substract ASCII char '0'
    sub dl,'0'

    ;substract the distance between the char '9' and ALPHABETIC char 'A' if nececery
    cmp dl,9
    jbe .ok
    sub dl,7
  .ok:
    ;add the converted digit to the result
    push pesi
    call AddEDXToQword

    inc pedi ;seek to next char in string
    jmp .loop
  .endx:
  call dleave
  ret (psize*3)
end if

;----------------------------------------------------
;                      StrToIntWithBaseAsTerminator
;Converts an ASCII format number to an integer, ending coversion by a zero-char of if char > base
;
;input:
;    1:source ASCII string
;    2:base for conversion
;
;output: (R/E)AX => integer
;        1:position of input source ASCII string + size of converted integer
;
;
;TODO:fix if base >10  part between cmp dl,9+7 and .ok (the chars 3A t/m 41 are read as 0 t/m 6)
;----------------------------------------------------
if used StrToIntWithBaseAsTerminator
StrToIntWithBaseAsTerminator:
  call denter
  mov pedi,pdword [pebp+(psize*2)] ;(R/E)DI => source ASCII string
  mov pebx,pdword [pebp+(psize*3)] ;(R/E)BX => base for conversion

  xor peax,peax  ;Zero output value
  xor pedx,pedx  ;Zero (R/E)DX
  .loop:
    ;Get digit from string and test on terminating zero byte
    mov dl,byte [pedi]
    test dl,dl
    jz .endx

    inc pedi ;seek to next char in string

    ;substract ASCII char '0'
    sub dl,'0'

    ;substract the distance between the char '9' and ALPHABETIC char 'A' if nececery
    cmp dl,9+7
    jbe .ok
    sub dl,7
  .ok:

    cmp dl,bl   ;if current digit => base  then end
    jae .endx

    ;multiply result by base
    imul peax,pebx

    ;add the converted digit to the result
    add peax,pedx
    jmp .loop

  .endx:
    mov pdword [pebp+(psize*2)],pedi ;replace source ASCII string with the end of the source string
    call dleave
    ret (psize*2)
end if

;----------------------------------------------------
;                      IntToStr
;Converts an integer to ASCII format
;
;input:
;    1:integer to convert
;    2:target ASCII string
;    3:base for conversion
;
;output: none
;
;----------------------------------------------------
if used IntToStr
IntToStr:
  call denter

  mov peax,pdword [pebp+(psize*2)]
  mov pedi,pdword [pebp+(psize*3)]
  mov pebx,pdword [pebp+(psize*4)]

  call internalIntToStr

  xor al,al
  stosb

  call dleave
  ret (psize*3)
end if

;----------------------------------------------------
;                  internalIntToStr
;Converts an integer to ASCII format
;
;input:
;    (R/E)AX => integer
;    (R/E)DI => string
;    ebx     => base
;
;output: (R/E)DI => pointer to last converted byte + 1
;
;Corupts:
;  edx, ecx, eax
;----------------------------------------------------
if used internalIntToStr
internalIntToStr:
  cld
  xor pecx,pecx  ;zero ammount of rest from divide instructions pushed on stack

  .new: ;divide integer by base until result is zero
    ;divide by base
    xor pedx,pedx
    div pebx

    ;push rest on stack and increment counter
    push pedx
    inc pecx

    ;is integer zero?
    test peax,peax
    jnz .new

  .loop: ;convert rest from divide function to ASCII-chars
    ;get rest from stack and add '0' as base
    pop peax
    add al,30h

    ;if char>'9' add 7 to make is an alphabetic character ('9'+7='A')
    cmp al,'9'
    jng .ok
    add al,7

  .ok:
    stosb ;store ASCII char to string
    loop .loop
ret
end if

;----------------------------------------------------
;                      IntToStrW
;Converts an integer to Unicode format
;
;input:
;    1:integer to convert
;    2:target ASCII string
;    3:base for conversion
;
;output: none
;
;----------------------------------------------------
if used IntToStrW
IntToStrW:
  call denter

  mov peax,pdword [pebp+(psize*2)]
  mov pedi,pdword [pebp+(psize*3)]
  mov pebx,pdword [pebp+(psize*4)]

  call internalIntToStrW

  xor ax,ax
  stosw

  call dleave
  ret (psize*3)
end if

;----------------------------------------------------
;                  internalIntToStrW
;Converts an integer to UNICODE format
;
;input:
;    (R/E)AX => integer
;    (R/E)DI => string
;    ebx     => base
;
;output: (R/E)DI => pointer to last converted byte + 2
;
;Corupts:
;  edx, ecx
;----------------------------------------------------
if used internalIntToStrW
internalIntToStrW:
  cld
  xor pecx,pecx  ;zero ammount of rest from divide instructions pushed on stack

  .new: ;divide integer by base until result is zero
    ;divide by base
    xor pedx,pedx
    div pebx

    ;push rest on stack and increment counter
    push pedx
    inc pecx

    ;is integer zero?
    test peax,peax
    jnz .new

  .loop: ;convert rest from divide function to ASCII-chars
    ;get rest from stack and add '0' as base
    pop peax
    add ax,30h

    ;if char>'9' add 7 to make is an alphabetic character ('9'+7='A')
    cmp ax,'9'
    jng .ok
    add ax,7

  .ok:
    stosw ;store UNICODE char to string
    loop .loop
ret
end if

;----------------------------------------------------
;                      IntQToStr
;Converts an 64-bit integer to ASCII format in 32-bit mode
;
;input:
;    1:pointer to 64-bit integer to convert
;    2:target ASCII string
;    3:base for conversion
;
;output: none
;
;----------------------------------------------------
if used IntQToStr & used is32
IntQToStr:
  call denter
        
  mov peax,pdword [pebp+(psize*2)]  ;pointer to Qword
  mov pedi,pdword [pebp+(psize*3)]  ;pointer to output buffer
  mov pebx,pdword [pebp+(psize*4)]  ;base

  call internalIntQToStr

  xor     al,al
  stosb

  call dleave
  ret (psize*3)
end if

;----------------------------------------------------
;                  internalIntQToStr
;Converts an 64-bit integer to ASCII format in 32-bit code
;
;input:
;    (R/E)AX => pointer to 64-bit integer
;    (R/E)DI => string
;    ebx     => base
;
;output: (R/E)DI => pointer to last converted byte + 1
;
;Corupts:
;  edx, ecx
;----------------------------------------------------
if used internalIntQToStr & used is32
internalIntQToStr:
  push dword [peax+0x4]
  push dword [peax]
  mov peax,pesp

  cld
  xor pecx,pecx  ;zero ammount of rest from divide instructions pushed on stack

  .new: ;divide integer by base until result is zero
    ;divide by base
    xor pedx,pedx
    push peax
    call DivQword

    ;push rest on stack and increment counter
    push pedx
    inc pecx

    ;is integer zero?
    cmp    dword [peax],0
    jnz     .new

    cmp    dword [peax+4],0
    jnz     .new

  .loop: ;convert rest from divide function to ASCII-chars
    ;get rest from stack and add '0' as base
    pop peax
    add al,30h

    ;if char>'9' add 7 to make is an alphabetic character ('9'+7='A')
    cmp al,'9'
    jng .ok
    add al,7

  .ok:
    stosb ;store ASCII char to string
    loop .loop

  add pesp,0x8
ret
end if

;----------------------------------------------------
;                      CreateOneString
;Creates a string on a new pointer out of different strings and integers.
;
;input:
;    1..*:List of strings and integers
;        {Type: 0x0=last item; 0x1=ASCII string; 0x2=Unicode string; 0x3=Decimal number; 0x31=Hexidecimal number; 0x10000=Copy caracters upper case; 0x20000=Free pointer;,
;         Pointer or integer}
;
;output: (R/E)AX: pointer to memory with new unicode string
;
;----------------------------------------------------
if used CreateOneString
CreateOneString:
  call denter

  lea pesi,pdword [pebp+psize*2]  ;get pointer to first item in list                ;
  xor pecx,pecx                   ;zero (R/E)CX, the size of the new string to create

  ;--
  lenItemsLoop:
    lodsp             ;Get type of current item in list
    and eax,0xFFFF    ;only get lower bytes
    mov edi,eax       ;EDI => type of item

    ;zero is last item in list
    test edi,edi
    jz endLenItemsLoop

    lodsp             ;Get pointer of current item in list

    ;1 is type ASCII string
    dec edi
    jz LenASCIString

    ;2 is type UNICODE string
    dec edi
    jz LenUNICODEString

    ;3 or 31 is type integer
    if used is32
      mov eax,(10+1)*2 ;maximum size of a integer in ASCII format in x86 mode
    else
      mov eax,(20+1)*2 ;maximum size of a integer in ASCII format in x64 mode
    end if
    jmp StrCounted

  LenUNICODEString:
    ;calculate size of Unicode string
    push peax
    call strlenW
    jmp StrCounted

  LenASCIString:
    ;calculate size of ASCII string
    push peax
    call strlen
    shl peax,1  ;double size (since the output will be an Unicode string)

  StrCounted:
    ;add size of item to total size of the new string.
    add pecx,peax
    add pecx,2 ;add space (needed if flag 0x40000 is set)
    jmp lenItemsLoop


endLenItemsLoop:
  inc pecx ;make sure the size of the new string is >0

  ;allocate the new string
  push pecx
  call GetMemLocation

  mov pebx,peax ;base address of the new string
  mov pedi,peax ;current position in string
  mov pedx,pebp ;pointer to stack (where list is located)

  if used is32
    push edx ;save EDX
  end if

  ;---
  DoStrCopyLoop:
    if used is32
      pop edx ;restore EDX
    end if

    add pedx,(psize*2)           ;seek to next item in the list
    mov peax,pdword [pedx]       ;Get type of current item in list

    if used is32
      push edx ;save EDX
    end if

    ;is this the last item in the list?
    test ax,ax
    jz EndStrCopyLoop

    mov pesi,pdword [pedx+psize] ;Get pointer of current item in list

    if used is32
      mov edx,eax  ;EDX => type
    else
      mov r12d,eax ;R12D => type
    end if

    ;1 is type ASCII string
    dec ax
    jz CopyASCIString

    ;2 is type UNICODE string
    dec ax
    jz CopyUNICODEString

    ;0x31 = hexidecimal, check for the '1'
    shr ax,4

    push pedx     ;save (R/E)DX
    push pebx     ;save (R/E)BX

    jz Decimal

    ;add hexidecimal sign '0x' to output string
    mov eax,0x00780030 ;L'0x'
    stosd

    mov peax,pesi ;(R/E)AX => integer to convert
    mov ebx,0x10           ;base = 16 (hexidecimal)
    call internalIntToStrW ;convert integer to Unicode string

    pop pebx ;restore (R/E)BX
    pop pedx ;restore (R/E)DX
    jmp nextDoStrCopyLoop

  ;--
  Decimal:
    mov peax,pesi ;(R/E)AX => integer to convert
    mov ebx,10             ;base = 10 (decimal)
    call internalIntToStrW ;convert integer to Unicode string

    pop pebx ;restore (R/E)BX
    pop pedx ;restore (R/E)DX
    jmp nextDoStrCopyLoop

  ;--
  CopyUNICODEString:
    test eax,0x20000
    jz dontPushPointerForFreeMemLocationUnicode
      push pesi ;push pointer for FreeMemLocation
    dontPushPointerForFreeMemLocationUnicode:

    push endCopyUNICODEString ;push return address

    test eax,0x10000 ;should the characters be transfered to high case?
    jnz InternalStrcpyWCaseHigh ;copy the Unicode string and set every character to high case
    ;copy the Unicode string
    jmp InternalStrcpyW

    endCopyUNICODEString:
    sub pedi,2 ;substract the zero byte
    jmp IsBitCallFreeMemLocatioSet

  ;--
  CopyASCIString:
    test eax,0x20000
    jz dontPushPointerForFreeMemLocation
      push pesi ;push pointer for FreeMemLocation
    dontPushPointerForFreeMemLocation:
    push endCopyASCIString ;push return address

    test eax,0x10000 ;should the characters be transfered to high case?
    jnz InternalStrcpyAWCaseHigh ;copy the Unicode string and set every character to high case
    ;copy the ASCII string
    jmp InternalStrcpyAW

    endCopyASCIString:
    sub pedi,2 ;substract the zero byte

    ;call FreeMemLocation is flag 0x20000 is set
    IsBitCallFreeMemLocatioSet:
      if used is32
        test edx,0x20000
      else
        test r12d,0x20000
      end if
      jz nextDoStrCopyLoop
        call FreeMemLocation

nextDoStrCopyLoop:
    ;Add extra zero-character at the end of this current item?
    if used is32
      test edx,0x40000
    else
      test r12d,0x40000
    end if
    jz dontAddZeroChar

      xor ax,ax
      stosw ;store zero char

dontAddZeroChar:
    jmp DoStrCopyLoop

  ;--
EndStrCopyLoop:
  if used is32
    pop edx ;restore EDX
  end if

  mov pdword [pedx],pebx  ;Save pointer of the new created string in the end of the list
  mov peax,pedx           ;(R/E)AX => pointer to the end of the list

  call dleave

  mov pesp,pdword [pesp]  ;(R/E)SP => return address
  xchg pdword [peax],pesp ;(R/E)SP => pointer to the new string, (d/q)word [(R/E)AX] => return address
  xchg pesp,peax          ;(R/E)SP => pointer to return address on stack, (R/E)AX => pointer to new created string
  ret
end if

;----------------------------------------------------
;                      CreateOneStringA
;Creates an ASCII string on a new pointer out of different strings and integers.
;
;input:
;    1...*:List of strings and integers
;        {Type: 0x0=last item; 0x1=ASCII string; 0x2=Unicode string; 0x3=Decimal number; 0x31=Hexidecimal number; 0x10000=Copy caracters upper case; 0x20000=Free pointer; 0x40000=Add zero-char behind;,
;         Pointer or integer}
;
;output: (R/E)AX: pointer to memory with new ASCII string
;
;----------------------------------------------------
if used CreateOneStringA
CreateOneStringA:
  call denter

  lea pesi,pdword [pebp+psize*2]  ;get pointer to first item in list                ;
  xor pecx,pecx                   ;zero (R/E)CX, the size of the new string to create

  ;--
  lenItemsLoopA:
    lodsp             ;Get type of current item in list
    and eax,0xFFFF    ;get only the lower bytes
    mov edi,eax       ;(R/E)DI => type of item

    ;zero is last item in list
    test pedi,pedi
    jz endLenItemsLoopA

    lodsp             ;Get pointer of current item in list

    ;1 is type ASCII string
    dec edi
    jz LenASCIStringA

    ;2 is type UNICODE string
    dec edi
    jz LenUNICODEStringA

    ;3 or 31 is type integer
    if used is32
      mov eax,(10+1)*2 ;maximum size of a integer in ASCII format in x86 mode
    else
      mov eax,(20+1)*2 ;maximum size of a integer in ASCII format in x64 mode
    end if
    jmp StrCountedA

  LenUNICODEStringA:
    ;calculate size of Unicode string
    push peax
    call strlenW
    shr peax,1  ;half size (since the output will be an ASCII string)
    jmp StrCountedA

  LenASCIStringA:
    ;calculate size of ASCII string
    push peax
    call strlen

  StrCountedA:
    ;add size of item to total size of the new string.
    add pecx,peax
    inc pecx ;add one byte (this for if flag 0x40000 is set)
    jmp lenItemsLoopA


endLenItemsLoopA:
  inc pecx ;make sure the size of the new string is >0

  ;allocate the new string
  push pecx
  call GetMemLocation

  mov pebx,peax ;base address of the new string
  mov pedi,peax ;current position in string
  mov pedx,pebp ;pointer to stack (where list is located)

  if used is32
    push edx ;save EDX
  end if

  ;---
  DoStrCopyLoopA:
    if used is32
      pop edx ;restore EDX
    end if

    add pedx,(psize*2)           ;seek to next item in the list
    mov peax,pdword [pedx]       ;Get type of current item in list

    if used is32
      push edx ;save EDX
    end if

    ;is this the last item in the list?
    test peax,peax
    jz EndStrCopyLoopA

    mov pesi,pdword [pedx+psize] ;Get pointer of current item in list

    if used is32
      mov edx,eax  ;EDX => type
    else
      mov r12d,eax ;R12D => type
    end if

    ;1 is type ASCII string
    dec ax
    jz CopyASCIStringA

    ;2 is type UNICODE string
    dec ax
    jz CopyUNICODEStringA

    ;0x31 = hexidecimal, check for the '1'
    shr ax,4

    push pedx     ;save (R/E)DX
    push pebx     ;save (R/E)BX

    jz DecimalA

    ;add hexidecimal sign '0x' to output string
    mov ax,'0x'
    stosw

    mov peax,pesi ;(R/E)AX => integer to convert
    mov ebx,0x10           ;base = 16 (hexidecimal)
    call internalIntToStr  ;convert integer to Unicode string

    pop pebx ;restore (R/E)BX
    pop pedx ;restore (R/E)DX
    jmp nextDoStrCopyLoopA

  ;--
  DecimalA:
    mov peax,pesi ;(R/E)AX => integer to convert
    mov ebx,10             ;base = 10 (decimal)
    call internalIntToStr  ;convert integer to Unicode string

    pop pebx ;restore (R/E)BX
    pop pedx ;restore (R/E)DX
    jmp nextDoStrCopyLoopA

  ;--
  CopyUNICODEStringA:
    test eax,0x20000
    jz dontPushPointerForFreeMemLocationUnicodeA
      push pesi ;push pointer for FreeMemLocation
    dontPushPointerForFreeMemLocationUnicodeA:

    push endInternalStrCpyWA ;push return address

    test eax,0x10000 ;should the characters be transfered to high case?
    jnz InternalStrcpyWACaseHigh ;copy the Unicode string and set every character to high case
    ;copy the Unicode string
    jmp InternalStrcpyWA

    endInternalStrCpyWA:
    dec pedi ;substract the zero byte
    jmp IsBitCallFreeMemLocatioSetA

  ;--
  CopyASCIStringA:
    test eax,0x20000
    jz dontPushPointerForFreeMemLocationA
      push pesi ;push pointer for FreeMemLocation
    dontPushPointerForFreeMemLocationA:

    ;copy the ASCII string
    StartCpyA:
      lodsb

      ;check if we need to replace lower case with upper case
      if used is32
        test edx,0x10000
      else
        test r12d,0x10000
      end if
      jz donSetCaseToHigh
        call SetHighCase

      donSetCaseToHigh:

      stosb
      test al,al
    jnz StartCpyA
    dec pedi ;substract the zero byte

    ;call FreeMemLocation is flag 0x20000 is set
    IsBitCallFreeMemLocatioSetA:
      if used is32
        test edx,0x20000
      else
        test r12d,0x20000
      end if
      jz nextDoStrCopyLoopA
        call FreeMemLocation

nextDoStrCopyLoopA:
    ;Add extra zero-character at the end of this current item?
    if used is32
      test edx,0x40000
    else
      test r12d,0x40000
    end if
    jz dontAddZeroCharA

      xor al,al
      stosb ;store zero char

dontAddZeroCharA:
  jmp DoStrCopyLoopA

  ;--
EndStrCopyLoopA:
  if used is32
    pop edx ;restore EDX
  end if

  mov pdword [pedx],pebx  ;Save pointer of the new created string in the end of the list
  mov peax,pedx           ;(R/E)AX => pointer to the end of the list

  call dleave

  mov pesp,pdword [pesp]  ;(R/E)SP => return address
  xchg pdword [peax],pesp ;(R/E)SP => pointer to the new string, (d/q)word [(R/E)AX] => return address
  xchg pesp,peax          ;(R/E)SP => pointer to return address on stack, (R/E)AX => pointer to new created string
  ret
end if