
;TV's server version 3.10
;Copyright (c) 2014 - 2019, Tycho Veltmeijer.
;All rights reserved.

;------------------------------------------------------------------------------
;The GetAPIVersion function will return the current API version
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    GetAPIVersion_c:
    StartProc 1

    mov rcx,retGetAPIVersion_c
    xchg qword [rsp],rcx
    jmp GetAPIVersion_asm

    retGetAPIVersion_c:
    sub rsp,1*psize
    EndProc 1
    jmp rcx
else
    GetAPIVersion_c:
end if
;---------
;asm implementation
;---------
GetAPIVersion_asm:
push pecx

mov eax,HIGHEST_SUPPORTED
mov ecx,LOWEST_SUPPORTED

;return the lowest supported version if flag is set
test dword [pesp+psize*2],GAV_LOWEST_SUPPORTED_VERSION
cmovnz eax,ecx

pop pecx
ret psize*1

;------------------------------------------------------------------------------
;The FindPluginDataStruct function will find the data members that belongs to this protocol
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    FindProtocolDataStruct_c:
    StartProc 2

    mov rcx,retFindProtocolDataStruct_c
    xchg qword [rsp],rcx
    jmp FindProtocolDataStruct_asm

    retFindProtocolDataStruct_c:
    sub rsp,2*psize
    EndProc 2
    jmp rcx
else
    FindProtocolDataStruct_c:
end if
;---------
;asm implementation
;---------
FindProtocolDataStruct_asm:
call denter

mov pecx,endFindProtocolDataStruct
call checkProtocolSession
;--;move internal protocol structure to edi/rdi
;--mov pedi,peax

;move psess to (R/E)DI
mov pedi,pdword [pebp+psize*2]
;move data to (R/E)DI
mov peax,pdword [pebp+psize*3]

loopThroughDataStructures:
  ;get the psess from this structure
  mov pecx,pdword [peax+psize*1]

  ;compair to the one we try to find
  cmp pecx,pedi
  je foundDataStruct

  ;skip to next data structure
  if used is32
   add peax,dword [peax]
  else
   mov ecx,dword [peax]
   add peax,rcx
  end if

  ;check if it was the last structure
  mov ecx,dword [peax]
  test ecx,ecx
jnz loopThroughDataStructures

xor eax,eax

;jmp endFindProtocolDataStruct

foundDataStruct:
;--add pedi,psize*2
;--mov peax,peax

endFindProtocolDataStruct:
call dleave
ret psize*2

;------------------------------------------------------------------------------
;The FormatDate function will create an integer from the input day, month and year.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    FormatDate_c:
    StartProc 3

    mov rcx,retFormatDate_c
    xchg qword [rsp],rcx
    jmp FormatDate_asm

    retFormatDate_c:
    sub rsp,3*psize
    EndProc 3
    jmp rcx
else
    FormatDate_c:
end if
;---------
;asm implementation
;---------
FormatDate_asm:
call denter

mov eax,dword [pebp+psize*4] ;day
mov ecx,dword [pebp+psize*3] ;month
shl ecx,5 ;move the month up 5 bits
or eax,ecx ;EAX + ECX
mov ecx,dword [pebp+psize*2] ;year
shl ecx,9 ;move the year up 9 bits
or eax,ecx ;EAX + ECX

call dleave
ret psize*3

;------------------------------------------------------------------------------
;The RecoverDate function will reverse the result of function FormatDate.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    RecoverDate_c:
    StartProc 2

    mov rcx,retRecoverDate_c
    xchg qword [rsp],rcx
    jmp RecoverDate_asm

    retRecoverDate_c:
    sub rsp,2*psize
    EndProc 2
    jmp rcx
else
    RecoverDate_c:
end if
;---------
;asm implementation
;---------
RecoverDate_asm:
call denter

;zero TIMEDATE structure
mov pedi,pdword [pebp+psize*3] ;(R/E)DI => date(TIMEDATE structure)
mov ecx,18/2
xor eax,eax
rep stosw
sub pedi,18 ;restore TIMEDATE pointer

mov eax,dword [pebp+psize*2]  ;eax = date (integer)
mov ecx,eax		      ;ecx = date (integer)
mov edx,eax		      ;edx = date (integer)

and edx,0x1F	    ;EDX => day
and ecx,0x1E0	    ;ECX => month>5
shr ecx,5	    ;ECX => month
and eax,0xFFFFFE00  ;EAX => year>9
shr eax,9	    ;EAX => year

stosd ;store year
mov word [pedi],cx ;store month
mov word [pedi+4],dx ;store day

call dleave
ret psize*2

;------------------------------------------------------------------------------
;The FormatTime function will create an integer from the input hours, minutes, seconds and miliseconds.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    FormatTime_c:
    StartProc 4

    mov rcx,retFormatTime_c
    xchg qword [rsp],rcx
    jmp FormatTime_asm

    retFormatTime_c:
    sub rsp,4*psize
    EndProc 4
    jmp rcx
else
    FormatTime_c:
end if
;---------
;asm implementation
;---------
FormatTime_asm:
call denter

mov eax,dword [pebp+psize*2] ;hours
mov ecx,dword [pebp+psize*3] ;minutes
shl ecx,5 ;move the minutes up 5 bits
or eax,ecx ;EAX + ECX
mov ecx,dword [pebp+psize*4] ;seconds
shl ecx,11 ;move the seconds up 11 bits
or eax,ecx ;EAX + ECX
mov ecx,dword [pebp+psize*5] ;miliseconds
shl ecx,17 ;move the miliseconds up 1 bits
or eax,ecx ;EAX + ECX

call dleave
ret psize*4

;------------------------------------------------------------------------------
;The RecoverDate function will reverse the result of function FormatDate.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    RecoverTime_c:
    StartProc 2

    mov rcx,retRecoverTime_c
    xchg qword [rsp],rcx
    jmp RecoverTime_asm

    retRecoverTime_c:
    sub rsp,2*psize
    EndProc 2
    jmp rcx
else
    RecoverTime_c:
end if
;---------
;asm implementation
;---------
RecoverTime_asm:
call denter

;zero TIMEDATE structure
mov pedi,pdword [pebp+psize*3] ;(R/E)DI => date(TIMEDATE structure)
mov ecx,18/2
xor eax,eax
rep stosw
sub pedi,8 ;(R/E)DI => pointer to hoor from TIMEDATE struct

mov eax,dword [pebp+psize*2]   ;eax = time (integer)
mov ecx,eax		       ;ecx = time (integer)
mov edx,eax		       ;edx = time (integer)
mov ebx,eax		       ;edx = time (integer)


and eax,0x1F	    ;EAX => hours
and ecx,0x7E0	    ;ECX => minutes>5
shr ecx,5	    ;ECX => minutes
and edx,0x1F800     ;EDX => seconds>11
shr edx,11	    ;EDX => seconds
and ebx,0x7FE0000   ;EBX => miliseconds>17
shr ebx,17	    ;EBX => miliseconds

stosw ;store hours
mov word [pedi],cx ;store minutes
mov word [pedi+2],dx ;store seconds
mov word [pedi+4],bx ;store milliseconds

call dleave
ret psize*2

;------------------------------------------------------------------------------
;The FormatDateTime function will create an long long from the input day, month, year, hours, minutes and seconds.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    FormatDateTime_c:
    StartProc 6

    mov rcx,retFormatDateTime_c
    xchg qword [rsp],rcx
    jmp FormatDateTime_asm

    retFormatDateTime_c:
    sub rsp,6*psize
    EndProc 6
    jmp rcx
else
    FormatDateTime_c:
end if


;---------
;asm implementation
;---------
FormatDateTime_asm:
call denter

mov eax,dword [pebp+psize*2] ;EAX = year
mov ebx,eax		     ;EBX = year
mov ecx,eax
sub ecx,1970

sub eax,1981		     ;1980 is the leap year, from here 8 leap days per 33 years
jc noLeapDaysAbove1980	     ;jump if onderflow

;calculate ammount of leap days
;--dec eax ;?

xor edx,edx
mov ecx,33
div ecx

;inc edx
;test eax,eax
;jz no1980Count

;  inc eax ;add one leap-day for 1980

;no1980Count:
mov esi,edx ;ESI = the not divided years
mov ecx,8
mul ecx     ;EAX = leap days since 1980
add eax,3;2 ;EAX = add leap-days between 1970-1980
mov edi,eax ;EDI = leap days since 1970 until year - year%33

mov ecx,esi ;ESI = the not divided years by 33

noLeapDaysAbove1980:
jecxz noYearsLeftToCalculateLeapDays

calculateLeapDays:
;int3
  mov eax,ebx ;EAX = year
  sub eax,ecx ;EAX = year to calculate if it is a leap year
  call isLeapYear
  add edi,eax ;add day if leap year
loop calculateLeapDays

noYearsLeftToCalculateLeapDays:
mov eax,ebx   ;EAX = year
sub eax,1970  ;1970 is the base year
mov ecx,365   ;365 days a year
xor edx,edx   ;clear edx
mul pecx      ;calculate the days in the years passed

if used is32
 mov pesi,dword [pebp+psize*8] ;get output value
 ;fill in the result of mul
 mov dword [pesi],eax
 mov dword [pesi+psize],edx

 ;days elappsed since 'year'
 mov ecx,pedi
 push pesi
 call AddToQword
else
  add pedi,peax ;days elappsed since 'year'
end if

;calculate if the current year is a leap year
mov eax,ebx
call isLeapYear
mov edx,eax ;leap year
mov ecx,dword [pebp+psize*3]
call calculateDaysInMonth

if used is32
 ;add days of the month(s) to the so far calculated days
 mov ecx,peax
 push esi
 call AddToQword
else
  add pedi,peax ;add days of the month(s) to the so far calculated days
end if

mov eax,dword [pebp+psize*4];days
dec eax ;the current day is still active

if used is32
 ;add day to the so far calculated days
 mov ecx,peax
 push esi
 call AddToQword
else
  add peax,pedi ;(R/E)AX = add day to the so far calculated days
end if


;calculate the ammount of hours in the days
mov ecx,24 ;24 hours in a day
if used is32
 push esi
 call MulQword
else
  mul pecx
end if


mov ecx,dword [pebp+psize*5] ;hours
if used is32
 ;hours
 push esi
 call AddToQword
else
  add peax,pecx 	       ;hours
end if


;calculate the ammount of minutes in the hours
mov ecx,60 ;60 minutes in a hour
if used is32
 push esi
 call MulQword
else
  mul pecx
end if

mov ecx,dword [pebp+psize*6] ;minutes
if used is32
 push esi ;minutes
 call AddToQword
else
  add peax,pecx 	       ;minutes
end if

;calculate the ammount of seconds in the minutes
mov ecx,60 ;60 seconds in a minute
if used is32
 push esi
 call MulQword
else
  mul pecx
end if

mov ecx,dword [pebp+psize*7] ;seconds
if used is32
 push esi ;seconds
 call AddToQword
else
  add peax,pecx 	       ;seconds
end if

if used is32
  xor eax,eax
  inc eax
end if

call dleave
if used is32
  ret psize*7
else
  ret psize*6
end if


;------------------------------------------------------------------------------
;This function will calculate if the year on EAX is a leap year
;input
;  EAX: year
;
;output
;  EAX: Leap year:1
;       Not leap year:0
;-----------------------------------------------------------------------------
isLeapYear:
push pecx
push pedx
push pebx
push pesi
push pedi

xor esi,esi
inc esi ;ESI = 1
mov edi,eax ;save EAX

xor edx,edx
mov ecx,400
div ecx
test edx,edx
cmovz eax,esi
jz endIsLeapYear ;is leap year

mov eax,edi ;restore eax
dec esi     ;esi = 0
xor edx,edx
mov ecx,100
div ecx
test edx,edx
cmovz eax,esi
jz endIsLeapYear ;is not leap year

mov eax,edi ;restore eax
xor edx,edx
mov ecx,4
div ecx
test edx,edx
mov eax,esi
jnz endIsLeapYear ;is not leap year

inc eax ;eax = 1

endIsLeapYear:
pop pedi
pop pesi
pop pebx
pop pedx
pop pecx
ret

;------------------------------------------------------------------------------
;This function will return the days in the month
;input
;  ECX: month
;  EDX: leap year
;
;output
;  EAX: days
;-----------------------------------------------------------------------------
calculateDaysInMonth:
xor eax,eax ;zero output
dec ecx
jz endCalculateDaysInMonth

add eax,31 ;januari is 31 days

dec ecx
jz endCalculateDaysInMonth

add eax,28  ;februari is 28 days
add eax,edx ;add leap day

dec ecx
jz endCalculateDaysInMonth

add eax,31 ;march is 31 days

dec ecx
jz endCalculateDaysInMonth

add eax,30 ;april is 30 days

dec ecx
jz endCalculateDaysInMonth

add eax,31 ;mei is 31 days

dec ecx
jz endCalculateDaysInMonth

add eax,30 ;juni is 30 days

dec ecx
jz endCalculateDaysInMonth

add eax,31 ;juli is 31 days

dec ecx
jz endCalculateDaysInMonth

add eax,31 ;august is 31 days

dec ecx
jz endCalculateDaysInMonth

add eax,30 ;sept is 30 days

dec ecx
jz endCalculateDaysInMonth

add eax,31 ;okt is 31 days

dec ecx
jz endCalculateDaysInMonth

add eax,30 ;nov is 30 days

;dec ecx
;jz endCalculateDaysInMonth

;add eax,31 ;dec is 31 days

endCalculateDaysInMonth:
ret

;------------------------------------------------------------------------------
;This function will return the month and the seconds that are left over
;input
;  EDX: seconds
;  EAX: leap year
;
;output
;  EDX: seconds left
;  EAX: month
;-----------------------------------------------------------------------------
calculateMonthFromSeconds:
push pesi ;save esi
push pecx ;save ecx

test eax,eax ;test on leap year
jz noLeapYearForMonthCalculation

  mov eax,86400 ;one day

noLeapYearForMonthCalculation:
xor ecx,ecx ;zero output month
inc ecx     ;start at januari

mov esi,2678400 ;31 days
sub edx,esi
jc endCalculateMonthFromSeconds

inc ecx 	;februari
mov esi,2419200 ;28 days
add esi,eax	;add leap day
sub edx,esi
jc endCalculateMonthFromSeconds

inc ecx 	;march
mov esi,2678400 ;31 days
sub edx,esi
jc endCalculateMonthFromSeconds

inc ecx 	;april
mov esi,2592000 ;30 days
sub edx,esi
jc endCalculateMonthFromSeconds

inc ecx 	;may
mov esi,2678400 ;31 days
sub edx,esi
jc endCalculateMonthFromSeconds

inc ecx 	;june
mov esi,2592000 ;30 days
sub edx,esi
jc endCalculateMonthFromSeconds

inc ecx 	;july
mov esi,2678400 ;31 days
sub edx,esi
jc endCalculateMonthFromSeconds

inc ecx 	;august
;mov esi,2678400 ;31 days
sub edx,esi
jc endCalculateMonthFromSeconds

inc ecx 	;sept
mov esi,2592000 ;30 days
sub edx,esi
jc endCalculateMonthFromSeconds

inc ecx 	;okt
mov esi,2678400 ;31 days
sub edx,esi
jc endCalculateMonthFromSeconds

inc ecx 	;nov
mov esi,2592000 ;30 days
sub edx,esi
jc endCalculateMonthFromSeconds

inc ecx 	;dec
jmp endCalculateMonthFromSecondsDec

endCalculateMonthFromSeconds:
add edx,esi ;restore seconds left

endCalculateMonthFromSecondsDec:
mov eax,ecx ;EAX = month

pop pecx
pop pesi
ret

;------------------------------------------------------------------------------
;The RecoverDateTime function will fill in the DateTime structure from a long long with the seconds since 1970.
;input
;  1:seconds since 1970 (qword)
;  2:output pointer
;    dword year
;    word month
;    word dayOfWeek
;    word day
;    word hour
;    word minutes
;    word seconds
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    RecoverDateTime_c:
    StartProc 2

    mov rcx,RecoverDateTime_c
    xchg qword [rsp],rcx
    jmp RecoverDateTime_asm

    retRecoverDateTime_c:
    sub rsp,2*psize
    EndProc 2
    jmp rcx
else
    RecoverDateTime_c:
end if
;---------
;asm implementation
;---------
RecoverDateTime_asm:
call denter

;zero TIMEDATE structure
mov pedi,pdword [pebp+psize*3] ;(R/E)DI => date(TIMEDATE structure)
mov ecx,18/2
xor eax,eax
rep stosw

mov pedx,pdword [pebp+psize*2] ;(R/E)DX => seconds since 1970
if used is32
  ;make a backup of seconds since 1970
  push dword [pedx+4]
  push dword [pedx]
  mov pedx,pesp
end if

mov peax,pedx		       ;(R/E)AX => backup of seconds since 1970
mov ebx,1970		       ;EBX = years

if used is32
  ;mov ecx,315532800 ;seconds 1970-1980 (3652*24*60*60)
  ;mov ecx,315619200 ;seconds 1970-1980 (3653*24*60*60)
  mov ecx,347155200 ;seconds 1970-1980 (3653*24*60*60)
  push pedx
  call SubToQword
  jnc atLeast10Years ;jump if no underflow

  ;restore seconds since 1970
  ;mov ecx,315532800 ;seconds 1970-1980 (3652*24*60*60)
  ;mov ecx,315619200 ;seconds 1970-1980 (3653*24*60*60)
  mov ecx,347155200 ;seconds 1970-1980 (3653*24*60*60)
  push pedx
  call AddToQword
  jmp calculateYearsSince1970
else
  ;sub pedx,315532800 ;seconds 1970-1980 (3652*24*60*60)
  ;sub pedx,315619200 ;seconds 1970-1980 (3653*24*60*60) 3653*24*60*60
  sub pedx,347155200 ;seconds 1970-1980 (3653*24*60*60)
  cmovc pedx,peax    ;restore seconds since 1970 if < then seconds 1970-1980
  jc calculateYearsSince1970
end if

atLeast10Years:
add pebx,10	 ;Add years 1970-1980

;Every 33 years there are 8 leap days: (365*33+8)*24*60*60 = 1041379200 seconds
if used is32
  push ebx ;save ebx

  ;divide by 33 years (in seconds)
  mov ebx,1041379200
  push pedx
  call DivQword

  ;(R/E)DX holds rest seconds
  ;esp holds years /33

  push pedx ;save rest seconds

  ;add the calculated years
  mov eax,dword [pesp+8]
  mov ecx,33
  mul ecx ;calculate the years

  pop pedx ;restore rest seconds
  pop ebx  ;restore ebx
  add ebx,eax

else
  mov rax,rdx	      ;RAX = seconds since 1980
  mov ecx,1041379200  ;33 years (in seconds)
  xor edx,edx	      ;clear RDX (32-bit operations will also clear 64-bit)
  div rcx

  ;RDX holds rest seconds
  ;RAX holds years /33

  push pedx ;save rest seconds

  mov rcx,33
  mul rcx	      ;calculate the years
  add rbx,rax	      ;add the calculated years

  pop pedx ;restore rest seconds
end if

;calculate the other years, by checking if it is a leap year or not
inc ebx
calculateYearsSince1970:
  mov eax,ebx ;EAX = year
  call isLeapYear
  mov ecx,31536000 ;seconds in a year without a leap year
  test eax,eax
  jz noLeapYear

    add ecx,86400  ;add leap-day

  noLeapYear:
  inc ebx     ;add one year
  sub edx,ecx ;substract the year in seconds
jnc calculateYearsSince1970 ;check on overflow

add edx,ecx ;restore last year (since the overflow)
dec ebx     ;restore year

mov pesi,pdword [pebp+psize*3] ;(R/E)SI => pointer to output buffer
mov dword [pesi],ebx ;store year in structure

;EDX = seconds left (for months, days, hours & seconds)
;EAX = 1(if leap year) or 0(if none-leap year)
call calculateMonthFromSeconds
mov word [pesi+4],ax ;store month in structure

;calulcate the day of the month
mov eax,edx   ;rest seconds
mov ecx,86400 ;one day in seconds
xor edx,edx   ;clear edx
div ecx       ;calculate the ammount of days
inc eax       ;increment by 1 (you don't start at day 0)
mov word [pesi+8],ax ;store day of the month in structure

;calulcate the hour of the day
mov eax,edx   ;rest seconds
mov ecx,3600  ;one hour in seconds
xor edx,edx   ;clear edx
div ecx       ;calculate the ammount of hours
mov word [pesi+10],ax ;store hours of the month in structure

;calulcate the minutes of the hour
mov eax,edx   ;rest seconds
mov ecx,60    ;one minute in seconds
xor edx,edx   ;clear edx
div ecx       ;calculate the ammount of minutes
mov word [pesi+12],ax ;store minutes

mov word [pesi+14],dx ;store seconds

if used is32
  sub pesp,8 ;delete copy of seconds since 1970
end if

;calculate the day of the week
mov pedx,pdword [pebp+psize*2] ;(R/E)DX => seconds since 1970
if used is32
  ;make a backup of seconds since 1970
  push dword [pedx+4]
  push dword [pedx]
  mov pedx,pesp
end if

mov ebx,3 ;WednesDay (31 dec 1969)
mov ecx,3

checkDayOfTheWeek:
  inc ebx ;incrment day of the week
  if used is32
    push ecx  ;save ecx

    mov ecx,86400
    push edx
    call SubToQword ;substract by 1 day

    pop ecx   ;restore ecx
  else
    sub rdx,86400 ;substract by 1 day
  end if
  jc foundDayOfTheWeek
loop checkDayOfTheWeek

mov ecx,604800 ;seconds in 1 week
mov peax,pedx	 ;(R/E)AX => seconds left
if used is32
  push peax
  call DivQword   ;divide by seconds in one week
else
  xor edx,edx
  div rcx	  ;divide by seconds in one week
end if
;EDX => seconds left for <7 day

mov eax,edx    ;EAX = seconds left
mov ecx,86400  ;seconds in 1 day
xor edx,edx    ;clear edx
div ecx        ;divide seconds by seconds in one day

mov ebx,eax


foundDayOfTheWeek:
mov word [pesi+6],bx ;store day of the week

call dleave
ret psize*2