
;TV's server version 3.10
;Copyright (c) 2014 - 2019, Tycho Veltmeijer.
;All rights reserved.

;entry dllmain

;a list with hashes of the tags commonly used if HTTP-traffic
;The list MUST be in order small to big
beginHashes:
  ;Size and hash of string 'host'
  ;db 4
  dd 0xD3981C34
  ;Size and hash of string 'date'
  ;db 4
  dd 0xCBA018A5
  ;Size and hash of string 'range'
  ;db 5
  dd 0xC53F3BE5
  ;Size and hash of string 'accept'
  ;db 6
  dd 0x498649E5
  ;Size and hash of string 'pragma'
  ;db 6
  dd 0xC56F3A59
  ;Size and hash of string 'cookie'
  ;db 6
  dd 0x614F0B57
  ;Size and hash of string 'referer'
  ;db 7
  dd 0x4EF67D25
  ;Size and hash of string 'if-match'
  ;db 8
  dd 0x46B6FA9F
  ;Size and hash of string 'if-range'
  ;db 8
  dd 0x46D7491C
  ;Size and hash of string 'user-agent'
  ;db 10
  dd 0xC641DE6B
  ;Size and hash of string 'connection'
  ;db 10
  dd 0xDC3970A1
  ;Size and hash of string 'content-type'
  ;db 12
  dd 0x22046B9C
  ;Size and hash of string 'authorization'
  ;db 13
  dd 0x5D0F7F59
  ;Size and hash of string 'cache-control'
  ;db 13
  dd 0x92D0490D
  ;Size and hash of string 'if-none-match'
  ;db 13
  dd 0xF68271DA
  ;Size and hash of string 'content-length'
  ;db 14
  dd 0x80B9A7EC
  ;Size and hash of string 'accept-language'
  ;db 15
  dd 0x4E11C0A9
  ;Size and hash of string 'accept-encoding'
  ;db 15
  dd 0xAC7E41C5
  ;Size and hash of string 'if-modified-since'
  ;db 17
  dd 0xF76443B3
  ;Size and hash of string 'if-unmodified-since'
  ;db 19
  dd 0xF1694AE5
dd 0 ;end of hashes

HOST_POS_DefaultHTTPVariables equ psize*3 ;Where 'Host' will be stored

;dllmain:
;call initGetSmallestPortionPageAllocation
;ret FunctionReturnFactor*3

;------------------------------------------------------------------------------
;This function will receive and parse the HTTP-header
;input:
;    1:ProtocolSession
;    2:ClientSession
;    3:maxHeaderSizeLimit
;    4:maxTimeToReceive
;    5:flags
;       0x1 RPF_DONT_PARSE
;       0x2 RPF_USE_HEADER_SIZE_LIMIT
;       0x4 RPF_USE_TIME_LIMIT
;    6:*error code
;
;output:
;    (R/E)AX => pointer to a HTTPSession structure
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    RecvAndParseHTTP_c:
    StartProc 6

    mov rcx,retRecvAndParseHTTP_c
    xchg qword [rsp],rcx
    jmp RecvAndParseHTTP

    retRecvAndParseHTTP_c:
    sub rsp,6*psize
    EndProc 6
    jmp rcx
else
    RecvAndParseHTTP_c:
end if
;---------
;asm implementation
;---------
RecvAndParseHTTP:
call denter

;--call initGetSmallestPortionPageAllocation ;<<<<---------delete als het een dll is

;receive the size of one page
call GetSmallestPortionPageAllocation
mov pecx,peax  ;ECX = buffer size
mov pebx,peax  ;EBX = size of one page
xor edx,edx    ;EDX = size of received data in buffer = 0

if used is64
  mov r12d,dword [rbp+psize*5]  ;R12d => time limit
  mov r13d,dword [rbp+psize*6]  ;R13d=>flags
  mov r15d,dword [pebp+psize*4] ;R15d=>maxHeaderSizeLimit

  test r13d,RPF_USE_TIME_LIMIT
  cmovz r12d,edx ;zero time limit if flag is not set
else
  test dword [pebp+psize*6],RPF_USE_TIME_LIMIT
  jnz timelimitPresent

  mov dword [pebp+psize*5],0 ;zero time limit

  timelimitPresent:
end if

push pecx
call GetMemLocation
mov pesi,peax  ;ESI = buffer

call getTickCount
if used is64
  mov r14,rax ;R14 => time the system is running
else
  push eax    ;dword [esp] => time computer is running
end if

;this loop will receive the HTTP header from the request
receiveHeader:
  ;do we have a limit in allowed size?
  if used is64
    test r13,RPF_USE_HEADER_SIZE_LIMIT
  else
    test dword [pebp+psize*6],RPF_USE_HEADER_SIZE_LIMIT
  end if
  jz noLimit

    ;check if we reached the size limit
    if used is64
      cmp r15,rdx
    else
      cmp pdword [pebp+psize*4],pedx
    end if
    mov eax,RPE_EXCEEDED_SIZE_LIMIT ;set error code on eax
    jc errProcessHTTPAtRecv ;End the function, since we reached the size limit

  noLimit:
  mov peax,pecx ;EAX = buffer size
  sub peax,pedx ;EAX = free space in buffer
  cmp pecx,pedx  ;buffer size - used size in buffer => free space in buffer
  jnz receiveData

  ;raise the size of the buffer with one page
  add pecx,pebx

  ;resize the buffer
  push pecx
  push pesi
  call ReAllocMemNoCheck
  mov pesi,peax ;set the next pointer to the buffer  \

  mov peax,pebx ;free space in the buffer is one page

receiveData:
  push peax ;push the free space of the buffer on stack

  push pesp ;&len for RecvData

  mov peax,pesi  ;(R/E)AX = pointer to buffer
  add peax,pedx  ;(R/E)AX = pointer to free space of buffer
  push peax ;buf
  if used is64
    push r12                 ;timeout
  else
    push dword [pebp+psize*5] ;timeout
  end if
  push RSF_RETURN_ON_FIRST_SUCCESSFUL_ATTEMPT ;flags
  push pdword [pebp+psize*3] ;ClientSession
  push pdword [pebp+psize*2] ;ProtocolSession
  call [RecvData]
  test eax,eax  ;check on error
  jz noErrorAtRecvData

    mov ecx,RPE_EXCEEDED_TIME_LIMIT  ;error code on ECX for this function
    cmp eax,CE_TIMEOUT               ;check error code from RecvData function
    cmovz eax,ecx                    ;set error from this function on eax
    jz errProcessHTTPAtRecv

    mov eax,RPE_CONNECTION_LOST ;set error from this function on eax
    jmp errProcessHTTPAtRecv


  noErrorAtRecvData:
  pop peax ;pop the received bytes

  add pedx,peax ;(R/E)DX = the new size of received bytes

  neg peax       ;(R/E)AX => -just received bytes
  add peax,pesi  ;(R/E)AX => the pointer to the buffer - just received bytes
  add peax,pedx  ;(R/E)AX => the pointer to the end of the buffer - just received bytes
  sub peax,4     ;(R/E)AX => the pointer to the last four bytes of the buffer

  cmp peax,pesi
  jae isOkPointer

  add peax,4

isOkPointer:
  mov pedi,pesi  ;(R/E)DI => the pointer to the buffer
  add pedi,pedx  ;(R/E)DI => the pointer to the end of the buffer
  add peax,4

  searchNewLine:
    ;inc peax  ;next byte
    sub peax,4

    ;is dubbel new line?
    cmp dword [peax],0x0A0D0A0D
    je foundNewLine

    add peax,5 ;go to next byte and add four bytes (that is what we are going to check
    ;check if end of buffer
    cmp pedi,peax
  jnc searchNewLine

  if used is64
    test r13d,RPF_CHECK_VALID_METHOD
  else
    test dword [pebp+psize*6],RPF_CHECK_VALID_METHOD
  end if
  jz dontCheckValidMethod

    ;only check method if at least 8 bytes have been received
    cmp pedx,9
    jc dontCheckValidMethod

      mov peax,pesi ;(R/E)AX => buffer
      call checkValidMethod
      test eax,eax
      mov eax,RPE_UNKNOWN_METHOD ;set error code on EAX
      jz errProcessHTTPAtRecv ;the method is not know, end the receive process

    dontCheckValidMethod:
  if used is64
    test r13d,RPF_USE_TIME_LIMIT
  else
    test dword [pebp+psize*6],RPF_USE_TIME_LIMIT
  end if
  jz receiveHeader ;there is no time limit

  call getTickCount
  if used is64
    sub rax,r14  ;tick count - previous tick count = time the coude runned
    add r14,rax ;R14 => result of last getTickCount call

    sub r12,rax  ;allowed time to run - time runned
    mov eax,RPE_EXCEEDED_TIME_LIMIT ;set error code on eax
    jc errProcessHTTPAtRecv ;jump if the allowed time <0

  else
    sub eax,dword [pesp]  ;tick count - previous tick count = time the code runned
    add dword [pesp],eax ;dword [esp] => result of last getTickCount call
    sub dword [pebp+psize*5],eax ;allowed time to run - time runned
    mov eax,RPE_EXCEEDED_TIME_LIMIT ;set error code on eax
    jc errProcessHTTPAtRecv ;jump if the allowed time <0

  end if

jmp receiveHeader

foundNewLine:
;from here it will disassemble the first rule with the request url, method and HTTP version
add peax,4 ;add the new line

sub pedi,peax ;(R/E)DI = size that does not belong to the HTTP-header (minth be a POST)
sub peax,pesi ;(R/E)AX => size of the header
mov pecx,peax ;(R/E)CX => size of the header

if used is64
  test r13d,RPF_CHECK_VALID_METHOD
else
  test dword [pebp+psize*6],RPF_CHECK_VALID_METHOD
end if
jz dontCheckValidMethod2

  mov peax,pesi ;(R/E)AX => buffer
  call checkValidMethod
  test eax,eax
  mov eax,RPE_UNKNOWN_METHOD
  jz errProcessHTTPAtRecv ;the method is not know, end this function

dontCheckValidMethod2:
call createHTTPSession
mov pdword [peax+psize*4],pesi ;set buffer
mov dword [peax+psize*7],ecx  ;header size
mov dword [(peax+psize*7)+4],edi  ;set rest size

mov pebx,peax ;(R/E)BX => HTTPSession

mov pedx,pdword [peax] ;(R/E)DX => defaultValues

if used is64
  mov dword [pebx+(psize*7)+8],r12d ;set timeLeft value
else
  pop eax   ;delete result of last getTickCount function from stack
  mov eax,dword [pebp+psize*5]  ;EAX = allowed time to run
  mov dword [pebx+(psize*7)+8],eax ;set timeLeft value
end if

if used is64
  test r13,RPF_DONT_PARSE
else
  test dword [pebp+psize*6],RPF_DONT_PARSE
end if
jnz endProcessHTTPSucces ;jump since we dont need to parse the header

;push 0
;push pesi
;call message

mov pdword [pedx+psize],pesi ;set Method
;find HTTP-Method
mov al,' '
mov pedi,pesi ;(R/E)DI => pointer to the buffer
;mov ecx,size ;ECX => already set
repnz scasb ;search for the space
mov eax,RPE_UNKNOWN_PROTOCOL
jnz errProcessHTTPAtHTTPSessionAllocated ;not found

mov byte [pedi-1],0 ;set zero byte where the space used to be

mov pdword [pedx+psize*2],pedi ;set URL
;find URL
mov al,' '
repnz scasb ;search for the space
mov eax,RPE_UNKNOWN_PROTOCOL
jnz errProcessHTTPAtHTTPSessionAllocated ;not found

mov byte [pedi-1],0 ;set zero byte where the space used to be

cmp dword [pedi],'HTTP' ;check if it is a HTTP request
;mov eax,RPE_UNKNOWN_PROTOCOL
jne errProcessHTTPAtHTTPSessionAllocated

add pedi,5 ;skip 'HTTP/'
sub ecx,5  ;adapt the counter
;mov eax,RPE_UNKNOWN_PROTOCOL
jc errProcessHTTPAtHTTPSessionAllocated ;jump if underflow

mov pdword [pedx],pedi ;set Version
;find URL
mov al,0x0D
repnz scasb ;search for the space
mov eax,RPE_UNKNOWN_PROTOCOL
jnz errProcessHTTPAtHTTPSessionAllocated ;not found

mov byte [pedi-1],0 ;set zero byte where the space used to be

inc pedi ;skip 0x0A byte
dec ecx  ;adapt the count
;mov eax,RPE_UNKNOWN_PROTOCOL
jz errProcessHTTPAtHTTPSessionAllocated ;jump if size is zero

;check if this is the end of the HTTP-request
cmp word [pedi],0x0A0D
je endLoopProcessHTTPHeader

if used is64
  mov r12,qword [pebx+psize] ;(R/E)SI => str_indexed_collection_header
end if

;(R/E)BX => HTTPSession
;(R/E)DX => defaultVariables
;    ECX => Size of bufer (R/E)DI
;(R/E)DI => buffer where thefirst HTTP-option starts
;R12 (x64) or dword [ebx+psize] (x86) => otherVariablesStringIndexedCollection
;This loop will deassemble the HTTP-header
loopProcessHTTPHeader:
  mov pesi,pedi ;save the start of the variablename

  ;check if this is the end of the HTTP-request
  cmp word [pedi],0x0A0D
  je endLoopProcessHTTPHeader

  ;find ':'
  mov al,':'
  repnz scasb ;search for the dubble dot
  jnz endLoopProcessHTTPHeader ;not found

  mov byte [pedi-1],0 ;set zero byte where the ':' used to be

  cmp byte [pedi],' '
  jne noSpaceToSkip

    mov al,' '
    repz scasb ;skip the spaces
    jz endLoopProcessHTTPHeader ;found space as last character

    dec pedi
    inc pecx

  noSpaceToSkip:
  push pedi ;push the start of the variable on stack

  ;find new line
  mov al,0x0D
  repnz scasb ;search for the space
  mov eax,RPE_UNKNOWN_PROTOCOL
  jnz errProcessHTTPAtHTTPSessionAllocated ;No new line found, this HTTP-header is corupted

  mov byte [pedi-1],0 ;set zero byte where the new line used to be

  ;generate the hash
  push pesi ;name of the variable
  call GenerateHashCaseInsensitiveSafe

  push pesi ;save the name of the variable on stack

  mov pesi,beginHashes

  ;search name in hash
  searchNameInHashes:
    cmp dword [pesi],eax ;check if the hashes match
    je foundHashInList

    add pesi,4
    cmp dword [pesi],0 ;check end of the list
  jne searchNameInHashes

  ;Did not found the name in the hashlist, add it to the indexed String Collection
  ;push data ;already done
  ;push index string ;already done
  if used is32
    push dword [pebx+psize]  ;push collection
  else
    push r12                ;push collection
  end if
  call AddIndexedStringItemWithHashNoCopy
jmp nextLoopProcessHTTPHeader

  foundHashInList:
  add pesp,psize ;remove the indexname from stack
  sub pesi,beginHashes ;calculate the hash position in the list
  if used is64
    shl pesi,1;a pointer is 8 bytes, not 4
  end if
  pop pdword [pedx+pesi+HOST_POS_DefaultHTTPVariables] ;set variable in default list

nextLoopProcessHTTPHeader:
  inc pedi ;skip         0x0A byte
  dec ecx  ;adapt the count
  jz endLoopProcessHTTPHeader ;jump if size is zero

  jmp loopProcessHTTPHeader ;next

endLoopProcessHTTPHeader:
;make a copy of the URL
push pdword [pedx+psize*2] ;URL
call copyStringA
mov pdword [pebx+psize*5],peax ;set on URL_buffer the copy of the URL
mov pedi,peax  ;(R/E)DI => copy of the URL

push pedi
call strlen
mov ecx,eax

;find question mark
mov al,'?'
repnz scasb ;search for the question mark
cmovz peax,pebx ;set HTTPSession as return value
jnz endLoopAssembleRequestURL ;not found

mov byte [pedi-1],0 ;zero the question mark
mov pesi,pdword [pebx+psize*2] ;(R/E)SI => optionsQuestionmarkStringIndexedCollection
jecxz endLoopAssembleRequestURL
;find the name and variable from the questionmark URL-string
loopAssembleRequestURL:
  mov pedx,pedi ;save the name of the varable

  ;find '='
  mov al,'='
  repnz scasb ;search for the question mark
  jnz endLoopAssembleRequestURL ;not found

  mov byte [pedi-1],0 ;set zero byte where the ':' used to be

  push pedi ;push the variable on stack
  push pedx ;push the name of the variable

  ;find '&'
  mov al,'&'
  repnz scasb ;search for the question mark
  jz foundAndSign

  xor edx,edx
  jmp doNotSetZeroByte

foundAndSign:
  ;save original byte
  xor edx,edx
  mov dl,byte [pedi-1]
  mov byte [pedi-1],0 ;set zero byte where the ':' used to be

doNotSetZeroByte:
  ;push pedi ;string       ;alredy pushed
  ;push pedx ;index string ;alredy pushed
  push pesi ;push collection
  call AddIndexedStringItemWithHashNoCopy

  test dl,dl
jnz loopAssembleRequestURL

endLoopAssembleRequestURL:
;-------------------------
;make a copy of the cookie value --if existst
mov pedx,pdword [pebx] ;(R/E)DX => DefaultHTTPVariables
mov pedx,pdword [pedx+psize*8] ;(R/E)DX => Cookie
test pedx,pedx
jz endloopDisassembleCookieString ;dont parse cookie string if not exists

push pedx ;Cookie string
call copyStringA
mov pdword [pebx+psize*6],peax ;set on cookie_buffer the copy of the Cookie string
mov pedi,peax  ;(R/E)DI => copy of the Cookie string

push pedi
call strlen
mov ecx,eax

mov pesi,pdword [pebx+psize*3] ;(R/E)SI => cookiesStringIndexedCollection
;find the name and variable from the cookie string
loopDisassembleCookieString:
  cmp byte [pedi],' '
  jne noSpaceToSkip2

    mov al,' '
    repz scasb ;skip the spaces
    jz endloopDisassembleCookieString ;found space as last character

    dec pedi
    inc pecx

  noSpaceToSkip2:

  mov pedx,pedi ;save the name of the varable
  ;find '='
  mov al,'='
  repnz scasb ;search for the question mark
  jnz endloopDisassembleCookieString ;not found

  mov byte [pedi-1],0 ;set zero byte where the ':' used to be

  push pedi ;push the variable on stack
  push pedx ;push the name of the variable

  ;find ';'
  mov al,';'
  repnz scasb ;search for the question mark
  jz foundNextSign

  ;push pdword [pesp]
  ;push pdword [pesp+psize*2]
  ;call message

  xor edx,edx
  jmp doNotSetZeroByteCookie

foundNextSign:
  ;save original byte
  xor edx,edx
  mov dl,byte [pedi-1]
  mov byte [pedi-1],0 ;set zero byte where the ':' used to be

doNotSetZeroByteCookie:
  ;push pedi ;string       ;alredy pushed
  ;push pedx ;index string ;alredy pushed
  push pesi ;push collection
  call AddIndexedStringItemWithHashNoCopy

  test dl,dl
jnz loopDisassembleCookieString

endloopDisassembleCookieString:

endProcessHTTPSucces:
xor eax,eax ;no error code
call setErrorCode
mov peax,pebx ;HTTP
jmp endProcessHTTP

;A small function to move error code from eax to pointer to *error parameter
setErrorCode:
mov pedi,pdword [pebp+psize*7] ;return error value
test pedi,pedi ;if the pointer is zero, we cant return an error return value
jz noErrorReturnCode

  mov dword [pedi],eax ;set error code

noErrorReturnCode:
ret

errProcessHTTPAtHTTPSessionAllocated:
call setErrorCode

push pebx
call FreeHttpSession
jmp errProcessHTTP

errProcessHTTPAtRecv:
call setErrorCode

push pesi
call FreeMemLocation

errProcessHTTP:
xor eax,eax ;zero return value

endProcessHTTP:
call dleave
ret psize*6

;------------------------------------------------------------------------------
;This function will check a pointer to start with the command GET, POST, HEAD, OPTIONS, PUT, DELETE, TRACE or CONNECT
;input:
;    (R/E)AX => pointer to buffer
;output:
;    (R/E)AX => true or false
;------------------------------------------------------------------------------
checkValidMethod:
push pesi ;save (R/E)SI
push pecx ;save (R/E)CX

mov pesi,peax ;(R/E)AX => pointer to buffer
xor ecx,ecx
inc ecx ;ecx => true

lodsd ;get four bytes from the buffer

cmp eax,'GET '
je endCheckValidMethod

cmp eax,'PUT '
je endCheckValidMethod

cmp eax,'POST'
je checkSpace

cmp eax,'HEAD'
je checkSpace

cmp eax,'OPTI'
je checkOPTIONS

cmp eax,'DELE'
je checkDELETE

cmp eax,'TRAC'
je checkTRACE

cmp eax,'CONN'
je checkCONNECT

jmp errEndCheckValidMethod


checkOPTIONS:
lodsd ;get next four bytes from the buffer
cmp eax,'ONS '
je endCheckValidMethod
jmp errEndCheckValidMethod

checkDELETE:
lodsw ;get next two bytes from the buffer
cmp ax,'TE'
je checkSpace
jmp errEndCheckValidMethod

checkTRACE:
lodsw ;get next two bytes from the buffer
cmp ax,'E '
je endCheckValidMethod
jmp errEndCheckValidMethod

checkCONNECT:
lodsd ;get next four bytes from the buffer
cmp eax,'ECT '
je endCheckValidMethod
jmp errEndCheckValidMethod

checkSpace:
lodsb
cmp al,' '
je endCheckValidMethod

errEndCheckValidMethod:
xor ecx,ecx ;ecx = false
jmp endCheckValidMethod

endCheckValidMethod:
mov eax,ecx ;set return value
pop pecx ;restore (R/E)CX
pop pesi ;restore (R/E)SI
ret

;------------------------------------------------------------------------------
;This function will create a HTTPSession structure
;input:
;    none
;output:
;    (R/E)AX => pointer to a HTTPSession structure
;------------------------------------------------------------------------------
createHTTPSession:
;create a collection for the cookiesStringIndexedCollection member
call CreateCollection
push peax

;create a collection for the optionsQuestionmarkStringIndexedCollection member
call CreateCollection
push peax

;create a collection for the otherVariablesStringIndexedCollection member
call CreateCollection
push peax

;-;create a DefaultHTTPVariables structure
;-push sizeOfDefaultHTTPVariables
;-call GetMemLocation
;-push peax ;push DefaultHTTPVariables on stack

;create a HTTPSession and DefaultHTTPVariables structure
push sizeOfHTTPSession+sizeOfDefaultHTTPVariables
call GetMemLocation
;fill in member defaultVariables
mov pdword [peax],peax
add pdword [peax],sizeOfHTTPSession
;pop pdword [peax] ;fill in member defaultVariables
pop pdword [peax+psize] ;fill in the member str_indexed_collection_header
pop pdword [peax+psize*2] ;fill in the member str_indexed_collection_questionmark_options
pop pdword [peax+psize*3] ;fill in the member cookiesStringIndexedCollection

ret

;------------------------------------------------------------------------------
;This function will free the HTTPSession structure
;input:
;    1:pointer to a HTTPSession structure
;output:
;    none
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    FreeHttpSession_c:
    StartProc 1

    mov rcx,retFreeHttpSession_c
    xchg qword [rsp],rcx
    jmp FreeHttpSession

    retFreeHttpSession_c:
    sub rsp,1*psize
    EndProc 1
    jmp rcx
else
    FreeHttpSession_c:
end if
;---------
;asm implementation
;---------
FreeHttpSession:
mov peax, pdword [pesp+psize] ;(R/E)AX => HTTPSession
test peax,peax
jz endFreeHttpSession
;--push pdword [peax]            ;push defaultVariables

push FreeDwordCollection                          ;argument FreeCollectionEx
push USE_FREE_ITEM_FUNCTION or REMOVE_INDEX_LIST  ;argument FreeCollectionEx
push pdword [peax+psize]      ;push string indexed collection (however, the pointers all point to buffer, so we can call FreeDwordCollection)

push FreeDwordCollection                          ;argument FreeCollectionEx
push USE_FREE_ITEM_FUNCTION or REMOVE_INDEX_LIST  ;argument FreeCollectionEx
push pdword [peax+psize*2]    ;push string indexed collection (however, the pointers all point to urlBuffer, so we can call FreeDwordCollection)

push FreeDwordCollection                          ;argument FreeCollectionEx
push USE_FREE_ITEM_FUNCTION or REMOVE_INDEX_LIST  ;argument FreeCollectionEx
push pdword [peax+psize*3]    ;push string indexed collection (however, the pointers all point to urlBuffer, so we can call FreeDwordCollection)

push pdword [peax+psize*4]    ;push buffer
push pdword [peax+psize*5]    ;push urlBuffer
push pdword [peax+psize*6]    ;push cookie_buffer

push peax  ;push HTTPSession
call FreeMemLocation

call FreeMemLocation ;free cookie_buffer
call FreeMemLocation ;free URL_buffer
call FreeMemLocation ;free buffer

call FreeCollectionEx ;free member string indexed collection (however, the pointers all point to buffer, so we can call FreeDwordCollection)
call FreeCollectionEx ;free member string indexed collection (however, the pointers all point to URL_buffer, so we can call FreeDwordCollection)
call FreeCollectionEx ;free member string indexed collection (however, the pointers all point to buffer, so we can call FreeDwordCollection)

;--call FreeMemLocation ;free defaultVariables

endFreeHttpSession:
ret psize

;------------------------------------------------------------------------------
;This function will convert bytes in format %xx to byte xx
;input:
;    1:input string
;    2:flags
;      0x1: ETBF_UNESCAPE_ADD_SIGN_AS_SPACE
;output:
;    input string
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    ConvertEscapedCharactersToBytes_c:
    StartProc 2

    mov rcx,retConvertEscapedCharactersToBytes_c
    xchg qword [rsp],rcx
    jmp ConvertEscapedCharactersToBytes

    retConvertEscapedCharactersToBytes_c:
    sub rsp,2*psize
    EndProc 2
    jmp rcx
else
    ConvertEscapedCharactersToBytes_c:
end if
;---------
;asm implementation
;---------
ConvertEscapedCharactersToBytes:
call denter

mov pedi,pdword [pebp+psize*2] ;(R/E)DI => input string
push pedi
call strlen
mov pecx,peax    ;ECX = length of input string
jecxz endConvertEscapedCharacterToBytes

loopSearchEscapedCharacter:
  jecxz endloopSearchEscapedCharacter
  ;Search for char '%'
  mov al,'%'
  repnz scasb
  jnz endloopSearchEscapedCharacter ;jump if reached the end of the string

  ;get first hex digit
  mov al,byte [pedi]

  call IsCharHexDigit ;check if char is a digit
  test ah,ah
  jz loopSearchEscapedCharacter ;find next % char

  mov dl,al ;store first digit in lower byte of word EDX
  ;get second hex digit
  mov al,byte [pedi+1]

  call IsCharHexDigit ;check if char is a digit
  test ah,ah
  jz loopSearchEscapedCharacter ;find next % char

  sal dl,4  ;move previous digit 4 bits up
  add dl,al ;add other digit to create the byte
  mov al,dl ;AL = DL
  dec pedi  ;previous position of the string
  stosb     ;store the unescaped byte
  mov byte [pedi],0 ;set zero byte

  mov pesi,pedi ;(R/E)SI => pointer to position of %+1
  add pesi,2    ;get position of next relevant char
  sub pecx,2    ;substract
  jc endloopSearchEscapedCharacter ;jump if underflow

  push pecx ;save current counter
  push pedi ;save current desination address

  rep movsb ;copy bytes behind the unescaped byte

  xor eax,eax
  stosb ;store zero ending byte

  pop pedi  ;restore counter
  pop pecx  ;restore desination address
jmp loopSearchEscapedCharacter

endloopSearchEscapedCharacter:
test dword [pebp+psize*3],ETBF_UNESCAPE_ADD_SIGN_AS_SPACE
jz endConvertEscapedCharacterToBytes

mov pedi,pdword [pebp+psize*2] ;(R/E)DI => input string
push pedi
call strlen
mov pecx,peax    ;ECX = length of input string

;Replace '+' with a space
loopSearchSpaceCharacter:
  jecxz endConvertEscapedCharacterToBytes
  ;Search for char '+'
  mov al,'+'
  repnz scasb
  jnz endConvertEscapedCharacterToBytes ;jump if reached the end of the string

  mov byte [pedi-1],' '
jmp loopSearchSpaceCharacter

endConvertEscapedCharacterToBytes:
mov peax,pdword [pebp+psize*2] ;return input string
call dleave
ret psize*2

;------------------------------------------------------------------------------
;This function will check if a assci byte is a hex digit
;input:
;    al:input byte
;output:
;    al => unescaped byte
;    ah => true or false
;------------------------------------------------------------------------------
IsCharHexDigit:
xor ah,ah ;zero return value

sub al,0x30
jc endIsCharHexDigit ;if al<'0' it is not a digit

inc ah ;return value is true

cmp al,10
jc endIsCharHexDigit ;if al>'0' & al=<'9', al is a digit

dec ah ;zero return value

sub al,0x11
jc endIsCharHexDigit ;if al<'A' it is not a digit

inc ah ;return value is true

cmp al,0x6
jc endIsCharHexDigitAplpha ;if al>'A' & al=<'F', al is a digit

dec ah ;zero return value

sub al,0x20
jc endIsCharHexDigit ;if al<'a' it is not a digit

inc ah ;return value is true

cmp al,0x6
jc endIsCharHexDigitAplpha ;if al>'a' & al=<'f', al is a digit

endIsCharHexDigitAplpha:
add al,0xA ;fix alpha digit

endIsCharHexDigit:
ret

;------------------------------------------------------------------------------
;This function will delete double slashes from a path ("//"=>"/" && "\\" => "\"
;input:
;    1:Pointer to path with string
;
;output:
;    none
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    DeleteDoubleSlashesInUrl_c:
    StartProc 1

    mov rcx,retDeleteDoubleSlashesInUrl_c
    xchg qword [rsp],rcx
    jmp DeleteDoubleSlashesInUrl

    retDeleteDoubleSlashesInUrl_c:
    sub rsp,1*psize
    EndProc 1
    jmp rcx
else
    DeleteDoubleSlashesInUrl_c:
end if
;---------
;asm implementation
;---------
DeleteDoubleSlashesInUrl:
call denter
mov pesi, pdword [pebp+psize*2] ;set source string pointer
push pesi
call strlen
mov pecx,peax ;set size
jecxz endLoopThroughString

loopThroughString:
  xor eax,eax
  lodsw ;read out two bytes
  cmp ax,'\\'
  je replaceSlash

  cmp ax,'//'
  je replaceSlash

  cmp ax,'/\'
  je replaceSlash

  cmp ax,'\/'
  je replaceSlash

  jmp nextLoopThroughString

  replaceSlash:

  mov pedi,pesi  ;(R/E)DI => pointer to current pos
  dec pedi       ;(R/E)DI => pointer to current pos - 1
  mov pedx,pecx  ;save counter
  mov pebx,pedi  ;save pointer -1 (since we need to recheck the new combination afther the byte move, it can be an other \\ or // combination).
  dec pecx

  rep movsb      ;move the string by one byte

  mov pesi,pebx  ;restore pointer to current pos
  mov pecx,pedx  ;restore counter
  ;inc pecx       ;since we did pointer -1, we also have to do counter +1

nextLoopThroughString:
  dec pesi ;set pointer 1 byte back (since we read 2 bytes a time)
loop loopThroughString

endLoopThroughString:
call dleave
ret psize

;------------------------------------------------------------------------------
;This function will delete the current directory symbols (./) in the path and decode the parent directory symbol (../)
;input:
;    1:Pointer to path with string
;
;output:
;    eax: 1=succesful, 0=there was a violation, one parent directory to mutch with no parent directory to cover
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    DecodeURL_c:
    StartProc 1

    mov rcx,retDecodeURL_c
    xchg qword [rsp],rcx
    jmp DecodeURL

    retDecodeURL_c:
    sub rsp,1*psize
    EndProc 1
    jmp rcx
else
    DecodeURL_c:
end if
;---------
;asm implementation
;---------
DecodeURL:
call denter

mov pesi,pdword [pebp+psize*2] ;Pointer to path with string
mov peax,pesi
inc peax     ;Pointer to path with string+1
cmp byte [pesi],'\'
cmove pesi,peax    ;if first char is '/', add one to the pointer
je skippedFirstSlash

cmp byte [pesi],'/'
cmove pesi,peax    ;if first char is '/', add one to the pointer
je skippedFirstSlash

jmp errDecodeURLPath;jump if the URL does not start with '/' or '\'

skippedFirstSlash:
push 0 ;will be used to determinate the last parent directory

push pesi
call strlen
mov pecx,peax ;ECX = size = counter
inc eax ;if ecx=0, then eax => true
jecxz endDecodeURLPath

loopScanOnCurAndParentDir:
  mov pedi,pesi ;save poisition of this parent directory
  xor eax,eax
  inc eax ;eax => true
  cmp pecx,3
  jc endDecodeURLPath ;jump if counter < 2

  lodsw ;read 2 bytes

  cmp ax,'..'
  jne isNotParentDirectory

  lodsb
  cmp al,'/'
  je isParentDirectory

  cmp al,'\'
  je isParentDirectory

  dec pesi ;restore pointer from lodsb

isNotParentDirectory:
  cmp ax,'./'
  je isCurrentDirectory

  cmp ax,'.\'
  je isCurrentDirectory

jmp findNextDirectory

  ;this is a parent directory symbol, get the directory above and
  isParentDirectory:
  ;mov peax,pesi  ;position of the next directory
  ;sub peax,pedi  ;-position of parent directory => size to delete from the path string

  pop pedi ;get pointer to previous directory
  test pedi,pedi
  jz errDecodeURLPath ;if there is no parent directory, raise error

  ;sub pecx,peax  ;adjust counter value with the size to delete
  sub pecx,3  ;substract ../ of ..\ from the counter
  jmp startMoveString

  ;this is a current directory symbol, delete it
  isCurrentDirectory:
  sub pecx,2    ;substract the counter with 2, since we are going to delete two characters ('./')

  startMoveString:
  mov pedx,pecx ;save counter
  mov pebx,pedi ;save pointer
  rep movsb     ;move the string to delete the two bytes
  mov pecx,pedx ;restore counter
  mov pesi,pebx ;restore pointer

  jmp nextLoopScanOnCurAndParentDir

  ;this is not a parent directory symbol of a current directroy symbol, find next directory name
  findNextDirectory:
  push pedi      ;push position of this directory on stack
  mov pesi,pedi  ;restore last position of the pointer to the directory

  ;loop through the path to find the next directory name
  loopFindNextDirectory:
    lodsb
    cmp al,'\'
    je endLoopFindNextDirectory
    cmp al,'/'
    je endLoopFindNextDirectory
  loop loopFindNextDirectory

  ;reached the end of the string with no errors, return true
  xor eax,eax
  inc eax ;eax = 1 (true)
  jmp endDecodeURLPath

  endLoopFindNextDirectory:
  nextLoopScanOnCurAndParentDir:
jmp loopScanOnCurAndParentDir


endLoopScanOnCurAndParentDir:

errDecodeURLPath:
xor eax,eax; EAX = 0 (error)

endDecodeURLPath:
call dleave
ret psize