
;TV's server version 3.10
;Copyright (c) 2014 - 2019, Tycho Veltmeijer.
;All rights reserved.

PLUGIN_VERSION equ ProductVersionPrefix, ".", FileVersion

CPO_NAME_AUTHOR equ -1
CPO_NAME_VERSION equ -2
CPO_NAME_KEEPALIVE equ 09
CPO_NAME_MAX_TIMEOUT equ 05

include 'include/function.inc'
include 'include/flags.inc'
include 'include/timedate.inc'       ;HTTP protocol
include 'include/math.inc'
include 'include_plugin_general/short_functions.inc'
include 'include_tvs_server_plugin_control_panel/page_parser.inc'
include 'include_tvs_server_plugin_control_panel/plugin_handle_connection.inc'

EXTRASIZE_LISTEN_AND_CLIENT_STRUCT equ psize



sizeSubportedProtocols equ endSubportedProtocols - subportedProtocols

include 'include_tvs_server_plugin_control_panel/plugin_bind.inc'
include 'include_plugin_general/list_functions.inc'
include 'include/collection.inc'
;include 'include_plugin_only/plugin_http_splitter.inc'
;include 'include_plugin_only/plugin_http_responder.inc'
;------------------------------
;-----GetProtocolNames---------
;-----__asm_exchange_tables----
;------------------------------
virtual at 0
call initHTTPProtocol
initHTTPProtocol:
end virtual
include 'include_plugin_general/__asm_exchange_tables_GetProtocolNames.inc'

cpyInternalProtocolstructure pointer 0
cpyInternalListenSessionstructureCollection pointer 0

generatedMenuProtocols pointer 0
generatedMenuListenSession pointer 0

currentDirectory pointer 0
lenCurrentDirectory pointer 0

;flags
;   WRITING 0x1
;reading
;    counter of how many reading operations there are currently. The flag WRITING may not be set if this is not zero
align 0x10 ;this is used by instructure cmpxchg8b and this instruction requiers alignment
cpyInternalProtocolstructureSignal:
reading dd 0
flags dd 0

main:
StartProc 3
call denter

  cmp dword [pebp+psize*3],1 ;DLL_PROCESS_ATTACH
  jne dontInitDLL

    call initGetSmallestPortionPageAllocation

    call getDirectoryByModule
    mov [currentDirectory],peax
    test peax,peax
    jz endMain ;error

    mov pesi,peax

    push peax
    call strlen
    mov [lenCurrentDirectory],peax

    mov ecx,eax
    mov pedi,element_path
    sub pedi,peax
    add pedi,260
    mov [pelement_path],pedi

    rep movsb


    call CreateCollection
    mov [cpyInternalListenSessionstructureCollection],peax

    call pushTVS_HTTP_PROTOCOL

      if used is32
        db 'tvs_server_http_protocol_x86.dll',0
      else
        db 'tvs_server_http_protocol_x64.dll',0
      end if

    pushTVS_HTTP_PROTOCOL:
    call loadmodule ;load HTTP protocol
    test peax,peax
    jz endMain

    mov pedx,peax
    xor ecx,ecx
    ;get pointers to functions
    mov esi,dword [pedx+0x3C] ;get offset of PE header
    cmp word [pedx+pesi+0x4+0x10],0x18  ;if SizeOfOptionalHeader<0x18 then BaseOfCode is not defined.
    cmovb eax,ecx
    jb endMain

    mov esi,dword [pedx+pesi+0x4+0x14+0x14] ;[Base + offset PE header + signature + fileheader + offset of BaseOfCode in optional header] = BaseOfCode
    test esi,esi
    cmovz eax,ecx
    jz endMain

    add pesi,pedx  ;pointer to first section with code

    ;check version dword
    lodsd ;get version
    cmp ax,0x0001
    jz copyFunctionsHTTP

    test ax,ax
    jz copyFunctionsHTTP

    test eax,eax
    cmovnz eax,ecx
    jnz endMain

  copyFunctionsHTTP:
    mov ecx,(endImportListHttp-beginImportListHttp)/psize
    mov  pedi,beginImportListHttp
    rep movsp ;copy the pointers

    cmp ax,0x1
    jae dontInitDLL

  mov [FreeMimeTypeFromExtension],0

dontInitDLL:
  xor eax,eax
  inc eax
endMain:
EndProc 3
call dleave
ret (FunctionReturnFactor*3)


;------------------------------------------------------------------------------
;The StartProtocol function need to initiate the protocol named in the ProtocolSession structure.
;------------------------------------------------------------------------------
StartProtocol:
StartProc 1
call denter

call PrepareElements



;Get the protocolName member from ProtocolSession
mov peax,pdword [pebp+(psize*2)]
mov peax,pdword[peax+(psize*1)]

;Do we init control panel
push controlPanel
push peax
call strcmp
test eax,eax
jne endStartProtocol

push V_TV_SERVER_101 ;version  (we need ShutdownServer function)
push EXTRASIZE_LISTEN_AND_CLIENT_STRUCT ;extraSize
push HTTP_psign ;pSign
push PT_LABORER or PT_SUPPORT_PSIGN or PT_CONTROLPANEL  ;type
push pdword [pebp+(psize*2)] ;psess
call [SetProtocolInformation]
test peax,peax
jz endStartProtocol

push httpControlPanelOptions  ;cpo
push pdword [pebp+(psize*2)] ;psess
call [SetControlPanelOptions]

push pdword [pebp+(psize*2)] ;psess
call [AcceptProtocol]

;set protocolsession
mov pecx,pdword [pebp+(psize*2)] ;psess
mov [psess_CONTROL_PANEL],pecx

;get a copy of all loaded protocols
push pdword [pebp+(psize*2)] ;psess
call [GetCopyOfInternalProtocolStructures]
mov pdword [cpyInternalProtocolstructure],peax

call GenerateMenuProtocols
mov [generatedMenuProtocols],peax

;register a function to be called when there is a new protocol loaded in to the memory
push ProtocolIsLoaded        ;funcion to register
push pdword [pebp+(psize*2)] ;psess
call [RegisterProtocolLoadedCallbackFunction]

;get ammount of listensessions
push pdword [pebp+psize*2] ;psess
call [CountListenSessions]
mov ecx,eax

jecxz endLoopLoadListenSessions

;get the ListenSession structures and there internal structures
loopLoadListenSessions:
  push 0

  push pesp           ;*lsess
  push pecx           ;id
  push pdword [pebp+psize*2] ;psess
  call [GetCopyOfInternalListenStructure]

  mov pebx,peax ;save the internal listen structure

  ;add index listensession
  ;push lsess ;push already done
  push [cpyInternalListenSessionstructureCollection]
  call AddDword

  ;add item internal listen session structure
  push pebx ;internal listen session structure
  push [cpyInternalListenSessionstructureCollection]
  call AddDword
loop loopLoadListenSessions

endLoopLoadListenSessions:
call GenerateMenuListenSessions
mov [generatedMenuListenSession],peax

;register a function to be called when there is a new ListenSession is created
push ListenSessionIsCreated  ;funcion to register
push pdword [pebp+(psize*2)] ;psess
call [RegisterListenSessionIsCreatedCallbackFunction]


;return
endStartProtocol:
call dleave
EndProc 1
ret (FunctionReturnFactor*1)

;------------------------------------------------------------------------------
;This function will be called when a new protocol is loaded in to the memory.
;This function can be implemented by Control Panel protocols and had to be registerd by the RegisterProtocolLoadedCallbackFunction function.
;input
;     1:psess of the new loaded protocol
;output:
;none
;------------------------------------------------------------------------------
ProtocolIsLoaded:
call denter

;try to lock the cpyInternalProtocolstructure linked list
TryWriteLockAgain:
  push 1
  call sleep

FirstTyrWriteLock:
  if used is32
    ;EDX(high) EAX(low) => Qword to compair [cpyInternalProtocolstructureSignal] with
    xor edx,edx
    xor eax,eax

    ;ecx(high) ebx(low) => Qword to set [cpyInternalProtocolstructureSignal] to
    xor ebx,ebx
    mov ecx,1 ;set write lock
    lock cmpxchg8b [cpyInternalProtocolstructureSignal]
  else
    xor eax,eax         ;RAX => Qword to compair [cpyInternalProtocolstructureSignal] with
    mov rcx,0x100000000 ;RCX => Qword to check [cpyInternalProtocolstructureSignal] to

    lock cmpxchg pdword [cpyInternalProtocolstructureSignal], rcx
  end if
jnz TryWriteLockAgain

;get a copy of all loaded protocols
push pdword [psess_CONTROL_PANEL] ;psess
call [GetCopyOfInternalProtocolStructures]
xchg pdword [cpyInternalProtocolstructure],peax      ;replace the original list

;release the previous list
push peax
call [ReleaseCopyOfInternalProtocolStructures]

;generate the protocol menu
call GenerateMenuProtocols
xchg [generatedMenuProtocols],peax

;free the privious protocols menu
push peax
call FreeMemLocation

mov [flags],0 ;clear write lock

call dleave
ret (FunctionReturnFactor*1)

;------------------------------------------------------------------------------
;This function will be called when a new ListenSession is created
;This function can be implemented by Control Panel protocols and had to be registerd by the RegisterListenSessionIsCreatedCallbackFunction function.
;input
;     1:lsess of the new created ListenSession
;     2:flags
;         0x1 LCCF_LSESS_UNLOADED The ListenSession, given on the lsess parameter, is unloaded and is no longer a valid value.
;output:
;none
;------------------------------------------------------------------------------
ListenSessionIsCreated:
StartProc 2
call denter

mov pebx,[cpyInternalListenSessionstructureCollection]  ;(R/E)BX => collection to work with
mov pedx,pdword [pebp+psize*2]                          ;(R/E)DX => lsess parameter

push FULL_LOCK
push pebx
call LockListAcces

test pdword [pebp+psize*3],LCCF_LSESS_UNLOADED
jz newListenSessionIsLoaded

  ;delete the listensession from the list (this functions locks and unlocks the collection
  push pedx                   ;name (lsess)
  push pebx                   ;collection
  call RemoveDwordByIndex

  ;delete the internal ListenStructure
  push peax
  call [ReleaseCopyOfInternalListenStructure]
  jmp CreateNewMenu

newListenSessionIsLoaded:

;add index listensession
push pedx  ;push already done
push pebx  ;collection
call AddDword

;get a copy of the InternalListenStructure
push pedx ;lsess
push pdword [psess_CONTROL_PANEL] ;psess
call [GetCopyOfInternalListenStructureByLsess]

;add item internal listen session structure
push peax ;internal listen session structure
push pebx
call AddDword

CreateNewMenu:
call GenerateMenuListenSessions
xchg [generatedMenuListenSession],peax

;free the privious protocols menu
push peax
call FreeMemLocation

push FULL_LOCK
push pebx
call ExternalUnlockListAccesEx

endListenSessionIsCreated:
call dleave
EndProc 2
ret (FunctionReturnFactor*2)